<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoDependente extends ProcessamentoSolicitacao {
    
    const TIPO_RELACIONAMENTO_FILHO       = 3;
    const TIPO_RELACIONAMENTO_TUTELADO    = 5;
    const TIPO_RELACIONAMENTO_PENSIONISTA = 6;
    
    const TIPODEPENDE_SALARIO_FAMILIA     = 1;
    const TIPODEPENDE_IMPOSTO_DE_RENDA    = 2;
    const TIPODEPENDE_PENSAO_JUDICIAL     = 3;
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrtiporelaciona    = $itens['NRTIPORELACIONA'] ?? null;
        $nmpessoa           = $itens['NMPESSOADEP']     ?? null;
        $idsexopessoa       = $itens['IDSEXOPESSOA']    ?? null;
        $dtnascifundparc    = $itens['DTNASCIFUNDPARC'] ?? null;
        $nrcpfdep           = $itens['NRCPFDEP']        ?  Utils::sanitizeCPF($itens['NRCPFDEP']) : null;
        $nrvinculom         = $this->getSolicitacao()->getNrvinculom();
        $nrorg              = Utils::getParamters()->getNrorg();
        
        $vinculom = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrorg"=> $nrorg,"nrvinculom" => $nrvinculom));
        $this->realizaCadastroPessoaDependente($nrvinculom, $vinculom->getNrpessoa(), $nmpessoa, null, null, null, null,
                null, $idsexopessoa, $dtnascifundparc, $nrtiporelaciona, null, $nrcpfdep, null);
    }
    
    public function rejeitar() {}
    
    public function realizaCadastroPessoaDependente($nrvinculom, $nrpessoavinc, $nmpessoa, $cdmunicipio, $sgestado, $cdpais, $nrnacionalid, $cdestacivil, $idsexopessoa,
                                                    $dtnascpessoa, $nrtiporelaciona, $dtfimrelacionamento, $cpfpessoa, $verificaCPF = false) {

        $nrorg = Utils::getParamters()->getNrorg();
        $dtmescompetencia = Utils::getParamters()->getDtmescomptencDiaCorrente();

        $idpessoafisica = 'S';
        $idinstituicao = 'N';
        $cdtipoinscricao = 'CPF';
        $cdtipoparceiro = 'PESSOA';

        try {
            Utils::getEntityManager()->beginTransaction();

            $pessoaVinc = Utils::getEntityManager()->getRepository(Repositories::GPE_PESSOA)->findOneBy(
                array('nrorg'=>$nrorg, 'nrpessoa'=>$nrpessoavinc)
            );
            $nrparcnegocio = $pessoaVinc->getNrparcnegocio();

            if ($dtnascpessoa){
                $dtnascpessoa = DateUtil::getDataDeString($dtnascpessoa, DateUtil::FORMATO_BRASILEIRO, true);
            }else{
                $dtnascpessoa = null;
            }    
            
            $parcnegocio = $this->novoParcNegocio(
                $nrorg, $nmpessoa, $nmpessoa, $cdtipoparceiro, $idpessoafisica, $idinstituicao, $cdtipoinscricao,
                '00000000000', $dtnascpessoa
            );

            $dtcompetencia = DateUtil::getDataDeString($dtmescompetencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            $pessoa = $this->novaPessoa($nrorg, $parcnegocio->getNrparcnegocio(), $dtcompetencia, $nmpessoa, null,
                null, $dtnascpessoa, $idsexopessoa, null, $cpfpessoa, null, null, null, null, null, null, null, null, $cdestacivil, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, $cdpais, $sgestado, $cdmunicipio,
                null, null, null, null, null, $nrnacionalid, null);

            // CADASTRO DE DEPENDENTE
            $this->novoRelacionaparc($nrorg, $nrtiporelaciona, $nrparcnegocio, $dtfimrelacionamento, $parcnegocio->getNrparcnegocio());

            $tipodepcadastro = array(3, 5, 6);
            if(in_array ($nrtiporelaciona , $tipodepcadastro)) {
                $dependentes = array();
                $dependentes["NRPARCDEPENDE"] = $parcnegocio->getNrparcnegocio();
                $dependentes["NRORG"] = $nrorg;
                $dependentes["NRTIPORELACIONA"] = $nrtiporelaciona;

                $dependentes = array($dependentes);
                $this->cadastroAutomaticoDependentes($nrorg, $nrvinculom, $dependentes);
            }
            
        
            Utils::getEntityManager()->commit();
            
            return $pessoa;
        } catch (\Exception $e) {
            Utils::getEntityManager()->rollback();
            throw $e;
        }
    }
    
    public function cadastroAutomaticoDependentes($nrorg, $nrvinculom, $dependentes) {
        try {
            
            foreach ($dependentes as $dependente) {
                $pessoaDependente = $this->getPessoaDependente($nrorg, $dependente["NRPARCDEPENDE"]);
                $this->criaDepvinculos($nrorg, $nrvinculom, $dependente["NRTIPORELACIONA"], $pessoaDependente->getNrpessoa());
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getPessoaDependente($nrorg, $nrparcdepende)
    {
        $pessoaDependente = $this->retornaPessoaPorParceiro($nrorg, $nrparcdepende);
        if (!is_object($pessoaDependente)) {
            $dtcompetencia = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(), DateUtil::FORMATO_BRASILEIRO, true);
            $arrayPessoa = $this->novaPessoa($nrorg, $nrparcdepende, $dtcompetencia, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null,
                null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null,
                null, null, null, null, null, null, null);
            $pessoaDependente = $arrayPessoa[0];
        }

        return $pessoaDependente;
    }
    
    public function criaDepvinculos($nrorg, $nrvinculom, $nrtiporelaciona, $nrpessoadep)
    {
        try {
            switch ($nrtiporelaciona) {
                case self::TIPO_RELACIONAMENTO_FILHO:
                    $this->criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                case self::TIPO_RELACIONAMENTO_PENSIONISTA:
                    $this->criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                case self::TIPO_RELACIONAMENTO_TUTELADO:
                    $this->criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoadep);
                    break;

                default:
                    break;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Filho (a)".
     */
    private function criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->validaIdadeMinCPF($nrpessoaDep, $nrorg);
            if ($this->validaSalarioFamilia($nrpessoaDep, $nrorg)) {
                $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_SALARIO_FAMILIA, $nrvinculom);
            }

            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaIdadeMinCPF($nrpessoaDep, $nrorg, $competencia = null){
        $idadeMinCPF = Utils::getEntityManager()->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor(
            Utils::getParamters()->getNrorg(),
            Utils::getParamters()->getNrorgPadrao(),
            Utils::getParamters()->getDtmescomptencDiaCorrente(),
            'IDADE_MIN_CPF_DEP'
        );

        if ($competencia == null) {
            $competencia = Utils::getParamters()->getDtmescomptencDiaCorrente();
        }
        $pessoaDependente = ProcessamentoSolicitacaoUtil::retornaHistoricoAtualPorPessoa($nrorg, $nrpessoaDep, $competencia);
        if ($pessoaDependente->getDtnascpessoa()){
            $dataNascDep = DateUtil::getDataDeString(substr($pessoaDependente->getDtnascpessoa(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $dataCalenda = DateUtil::getDataDeString(substr($competencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $idadeDep = $dataCalenda->diff($dataNascDep)->y; 
            if(!$pessoaDependente->getNrcpfpessoa() && $idadeDep >= $idadeMinCPF )
                throw new \Exception('CPF não está cadastrado para Dependente '.$pessoaDependente->getNmpessoa(),10);
        }else {
             throw new \Exception('Data de nascimento não está cadastrada para Dependente '.$pessoaDependente->getNmpessoa(),10);
        }
        
    }
    
    // public function retornaHistoricoAtualPorPessoa($nrorg, $nrpessoa, $dtmescompetenc){

    //     $qb = Utils::getEntityManager()->createQueryBuilder();
    //     $gpePessoah = Repositories::GPE_PESSOAH;
    //     $pessoah = $qb
    //         ->select('p')
    //         ->from($gpePessoah, 'p')
    //         ->where('p.nrorg = :nrorg')
    //         ->andWhere('p.nrpessoa = :nrpessoa')
    //         ->andWhere('p.dtmescompetenc <= :dtmescompetenc')
    //         ->orderBy('p.dtmescompetenc', 'DESC')
    //         ->setParameters(array('nrorg' => $nrorg, 'nrpessoa' => $nrpessoa, 'dtmescompetenc' => $dtmescompetenc))
    //         ->getQuery()
    //         ->getResult();
    //     return current($pessoah);
    // }
    
    public function validaSalarioFamilia($nrpessoaDep, $nrorg, $competencia = null)
    {
        if ($competencia == null) {
            $competencia = Utils::getParamters()->getDtmescomptencDiaCorrente();
        }

        $pessoah = ProcessamentoSolicitacaoUtil::retornaHistoricoAtualPorPessoa($nrorg, $nrpessoaDep, $competencia);

        if (is_object($pessoah)) {
            if ($pessoah->getDtnascpessoa()) {
                $dtnascpessoa = $pessoah->getDtnascpessoa();
                //$dtnascpessoa = DateUtil::getDataDeString($dtnascpessoa); 
                $dataAtual = DateUtil::getDataAtual();
                if ($dataAtual->diff($dtnascpessoa)->y > 14) {
                    return false;
                }
            }
        }

        return true;
    }
    
    /**
     * Insere um novo registro na tabela fpa_depvinculo. Este metodo somente criara um novo se nao houver nenhum outro
     * fazendo a mesma dependencia.
     */
    public function insereDepVinculo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom)
    {
        if ($this->existeDependencia($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom) === false) {
            try {
                $this->novoDepVinculo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom);
            } catch (\Exception $e) {
                throw $e;
            }
        }
    }
    
    public function existeDependencia($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom)
    {
        try {
            $depvinculo = Utils::getEntityManager()->getRepository(Repositories::FPA_DEPVINCULO)->findOneBy(
                array('nrorg'=>$nrorg, 'nrpessoa'=> $nrpessoa, 'nrtipodepende'=>$nrtipodepende, 'nrvinculom'=>$nrvinculom)
            );
            return is_object($depvinculo) > 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function novoDepVinculo($nrorg, $nrpessoa, $nrtipodepende, $nrvinculom)
    {
        $fpaDepvinculo = Repositories::FPA_DEPVINCULO;
        $depvinculo = new $fpaDepvinculo;

        $depvinculo->setNrorg($nrorg);
        $depvinculo->setNrpessoa($nrpessoa);
        $depvinculo->setNrtipodepende($nrtipodepende);
        $depvinculo->setNrvinculom($nrvinculom);
        $depvinculo->setNrdepvinculo(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'FPA_DEPVINCULO', 1, 12, true));

        $depvinculo->setDtinclusao(DateUtil::getDataAtual());
        $depvinculo->setNrorginclusao(Utils::getParamters()->getNrorg());
        $depvinculo->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $depvinculo->setDtultatu(DateUtil::getDataAtual());
        $depvinculo->setNrorgultatu(Utils::getParamters()->getNrorg());
        $depvinculo->setCdoperultatu(Utils::getParamters()->getCdoperador());

        Utils::getEntityManager()->persist($depvinculo);
        Utils::getEntityManager()->flush($depvinculo);
        return $depvinculo;
    }
    
    /**
     * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Pensionista".
     */
    private function criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_PENSAO_JUDICIAL, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Tutelado". 
     */
    private function criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoaDep)
    {
        try {
            $this->validaIdadeMinCPF($nrpessoaDep, $nrorg);
            $this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function novoParcNegocio($nrorg, $nmprincipalparc, $nmsecundariparc, $cdtipoparceiro, $idpessoafisica, $idinstituicao,
                         $cdtipoinscricao, $nrinscricaoparc, $dtnascifundparc)
    {
        $parcnegocio = Repositories::PARCNEGOCIO;
        $parceiro = new $parcnegocio;
        
        $idativo = 'S';
        $parceiro->setNrorg($nrorg);
        $parceiro->setIdativo($idativo);
        $parceiro->setNmprincipalparc($nmprincipalparc);
        $parceiro->setNmsecundariparc($nmsecundariparc);
        $parceiro->setCdtipoparcprincipal($cdtipoparceiro);
        $parceiro->setIdpessoafisica($idpessoafisica);
        $parceiro->setIdinstituicao($idinstituicao);
        $parceiro->setCdtipoinscricao($cdtipoinscricao);
        $parceiro->setNrinscricaoparc($nrinscricaoparc);
        $parceiro->setDtnascifundparc($dtnascifundparc);

        $parceiro->setDtinclusao(DateUtil::getDataAtual());
        $parceiro->setNrorginclusao(Utils::getParamters()->getNrorg());
        $parceiro->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $parceiro->setDtultatu(DateUtil::getDataAtual());
        $parceiro->setNrorgultatu(Utils::getParamters()->getNrorg());
        $parceiro->setCdoperultatu(Utils::getParamters()->getCdoperador());

        $parceiro->setNrparcnegocio(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'PARCNEGOCIO', 1, 12, true));

        Utils::getEntityManager()->persist($parceiro);
        Utils::getEntityManager()->flush($parceiro);
        //\ParceriaEntity::incluiParceria($this->nrorg, $this->parceiro->getNrparcnegocio(), $cdtipoparceiro, $idativo);

        return $parceiro;
    }
    
    public function novoRelacionaparc($nrorg, $nrtiporelaciona, $nrparcnegocio, $dtfimrelacionamento, $nrparcnegrelac)
    {
        $relacionaparc = Repositories::RELACIONAPARC;
        $relacionaparc = new $relacionaparc;

        $relacionaparc->setNrorg($nrorg);
        $relacionaparc->setDtinclusao(DateUtil::getDataAtual());
        $relacionaparc->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $relacionaparc->setNrorginclusao(Utils::getParamters()->getNrorg());
        $relacionaparc->setDtultatu(DateUtil::getDataAtual());
        $relacionaparc->setCdoperultatu(Utils::getParamters()->getCdoperador());
        $relacionaparc->setNrorgultatu(Utils::getParamters()->getNrorg());
        $relacionaparc->setNrparcnegocio($nrparcnegocio);
        $relacionaparc->setNrtiporelaciona($nrtiporelaciona);
        $relacionaparc->setNrparcnegrelac($nrparcnegrelac);
        //$relacionaparc->setDtfimrelacionamento($dtfimrelacionamento);
        $relacionaparc->setIdativo('S');
        $relacionaparc->setNrrelacionaparc(NovoCodigo::geraCodigo($nrorg, 'RELACIONAPARC', 1, 10, true));

        Utils::getEntityManager()->persist($relacionaparc);
        Utils::getEntityManager()->flush($relacionaparc);
        return $relacionaparc;
    }
    
    public function retornaPessoaPorParceiro($nr_org, $nr_parcnegocio){
        $pessoa = Utils::getEntityManager()->getRepository(Repositories::GPE_PESSOA)->findOneBy(
            array('nrorg'=>$nr_org, 'nrparcnegocio'=>$nr_parcnegocio)
        );

        /*if (is_object($pessoa) === true) {
            $dao = $pessoa->getDao();
            $historico = self::_retornaUltimoHistorico($nr_org, $dao->NRPESSOA);
            $pessoa->historico = $historico;
        }*/

        return $pessoa;
    }
    
    public function novaPessoa(
        $nrorg, $nrparcnegocio, $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento,
        $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS,
        $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento,
        $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao,
        $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $nrCategoriaMilitar,
        $nrCertificadoReservista, $dsCertificadoReservista, $dtCertificadoReservista,
        $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado,
        $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG, $dsLocalExpedicaoRG, $dtExpedicaoRG,
        $nrNacionalidade, $nrGrauInstrucao, $pronometrat = null, $nranochegapais = null,
        $nrdocuestrang = null, $idpesnaturaliza = null, $dtnaturalizapes = null, $idposvistoperm = null,
        $dtpermavisto = null, $nrconselhoreg = null, $nrinscrconsreg = null, $dtexconsreg = null,
        $sgufconsreg = null, $cdtpbaixapag = null, $nrlivrocertnasc = null, $nrfolhacertnasc = null, $dscomplcertnasc = null,$nrregctpspes = null,
        $nrclassestrang = null, $dsexpdocestr = null, $dtchegadapais = null, $idcotapcd = null
    )
    {
        $pessoa = $this->novoMestrePessoa($nrorg, $nrparcnegocio);
        $pessoah = ProcessamentoSolicitacaoUtil::novoHistoricoPessoa(
            $nrorg, $pessoa->getNrpessoa(), $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento,
            $dtNascimento, $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS,
            $dtPIS, $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento,
            $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao,
            $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $nrCategoriaMilitar, $nrCertificadoReservista,
            $dsCertificadoReservista, $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca,
            $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG,
            $dsLocalExpedicaoRG, $dtExpedicaoRG, $nrNacionalidade, $nrGrauInstrucao, $pronometrat, $nranochegapais, $nrdocuestrang,
            $idpesnaturaliza, $dtnaturalizapes, $idposvistoperm, $dtpermavisto, $nrconselhoreg, $nrinscrconsreg, $dtexconsreg,
            $sgufconsreg, $cdtpbaixapag, $nrlivrocertnasc, $nrfolhacertnasc, $dscomplcertnasc,$nrregctpspes,
            $nrclassestrang, $dsexpdocestr, $dtchegadapais, $idcotapcd 
        );
        return array($pessoa, $pessoah);
    }
    
    public function novoMestrePessoa($nrorg, $nrparcnegocio) {
        $gpePessoa = Repositories::GPE_PESSOA;
        $pessoa = new $gpePessoa;

        $pessoa->setNrorg($nrorg);
        $pessoa->setNrparcnegocio($nrparcnegocio);

        $pessoa->setDtinclusao(DateUtil::getDataAtual());
        $pessoa->setNrorginclusao(Utils::getParamters()->getNrorg());
        $pessoa->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $pessoa->setDtultatu(DateUtil::getDataAtual());
        $pessoa->setNrorgultatu(Utils::getParamters()->getNrorg());
        $pessoa->setCdoperultatu(Utils::getParamters()->getCdoperador());

        $pessoa->setNrpessoa(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_PESSOA', 1, 12, true));

        Utils::getEntityManager()->persist($pessoa);
        Utils::getEntityManager()->flush($pessoa);
        return $pessoa;
    }
    
    public function getPronomeTratamento($cdEstadoCivil,$idSexoPessoa) {
        if ($idSexoPessoa == 'M'){
            $pronomeTratamento = 'Sr.';
            return $pronomeTratamento;
        }
        else if ($idSexoPessoa == 'F' && $cdEstadoCivil == 'C'){
            $pronomeTratamento = 'Sra.';
            return $pronomeTratamento;
        }
        else {
            $pronomeTratamento = 'Srta.';
            return $pronomeTratamento;
        }
    }
}