<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;

use Util\DateUtil;
use Util\NovoCodigo;
use Ems\Util\Utils;
use Ems\Services\MarcacaoService;

// MENSAGENS
define('messagesMarcacaoRepository', [
    'ERRO_VINCULO_NAO_ENCONTRADO'  => 'Vínculo não foi encontrado!',
    'ERRO_PARAMORG_NAO_ENCONTRADO' => 'Parâmetros da estrutura não encontrado!'
]);

class MarcacaoRepository {
    protected $connection;

    const TIPO_VINC_CLT = 1;
    const VINC_EMPREG_APRENDIZ = 9;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getMarcacao(int $nrOrg, int $nrMarcacao) {
        $params = [
            'P_NRORG'      => $nrOrg,
            'P_NRMARCACAO' => $nrMarcacao
        ];

        $sql = <<<SQL
            SELECT GPE_MARCACAOPONTO.*,
                   TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY') DIA,
                   TO_CHAR(DTHRMARCACAOPONTO, 'HH24:MI') HORA_MIN
            FROM GPE_MARCACAOPONTO
            WHERE NRORG = :P_NRORG
              AND NRMARCACAOPONTO = :P_NRMARCACAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacao = $std->fetch();

        return $marcacao;
    }

    public function getVinculosGestor($params) {
        $sql = <<<SQL
            SELECT
                GPE_VINCULOM.NRVINCULOM,
                GPE_VINCULOM.NMVINCULOM,
                GPE_VINCULOM.NRVINCULOM || ' | ' || GPE_VINCULOM.NMVINCULOM AS NRNMVINCULO,
                GPE_VINCULOM.DTADMISSAOVINC,
                GPE_VINCULOM.DTRESCISAOVINC,
                GPE_VINCULOM.NRTIPOVINCULOM,
                GPE_VINCULOM.NRPESSOA,
                GPE_VINCULOM.NRORG,
                GPE_VINCULOM.NRVINCULOEMPREG,
                GPE_VINCULOH.NRESTRUTLEGAL,
                GPE_VINCULOH.NRESTRUTGEREN,
                PH.NRCPFPESSOA,
                PH.NMPESSOA,
                PA.NRPARCNEGOCIO,
                ESTV.DTFIMESTABVINC,
                MTVES.DSMOTIVOESTAB,
                PSCT.DTBASESINDICAL,
                PCN.HASHFACIAL,
                SFH.IDAFASTSITUFUNC,
                CASE
                    WHEN PSCT.DTBASESINDICAL IS NOT NULL THEN
                        TO_DATE(TO_CHAR(PSCT.DTBASESINDICAL, 'DD/MM') || TO_CHAR(SYSDATE, 'YYYY'), 'DD/MM/YYYY')
                    ELSE NULL
                END AS DTBASESINDICAL_ATUAL,
                CASE
                    WHEN PSCT.DTBASESINDICAL IS NOT NULL THEN
                        TO_DATE(TO_CHAR(PSCT.DTBASESINDICAL - 31, 'DD/MM') || TO_CHAR(SYSDATE, 'YYYY'), 'DD/MM/YYYY')
                    ELSE NULL
                END AS DTBASESINDICAL_ATUAL_MIN,
                CASE
                    WHEN GPE_VINCULOH.NRESCALATRABM IS NOT NULL THEN
                        GPE_VINCULOM.NRVINCULOM || ' | ' || GPE_VINCULOM.NMVINCULOM || ' (' || ETH.NMESCALATRABH || ')'
                    ELSE
                        GPE_VINCULOM.NRVINCULOM || ' | ' || GPE_VINCULOM.NMVINCULOM
                END AS NMVINCULOM_NMESCALATRABH,
                MARCACOES_DIA.HORARIOS_MARCACOES_DIA_ATUAL,
                CASE
                    WHEN SFH.IDAFASTSITUFUNC = 'S' OR (
                        GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                        AND GPE_VINCULOM.DTRESCISAOVINC <= LAST_DAY(:P_DTMESCOMPETENC)
                    ) THEN 'S'
                    ELSE 'N'
                END AS IDDESLGOURESCD,
                CASE
                    WHEN ESTV.NRALTESTABVINC IS NOT NULL THEN 'S'
                    ELSE 'N'
                END AS IDSITUACAOESTAB,
                CASE
                    WHEN PH.IDCOTAPCD = 'S' OR GPE_VINCULOM.NRVINCULOEMPREG = 9 THEN 'S'
                    ELSE 'N'
                END AS IDCOTAPCDAPRENDIZ,
                CASE
                    WHEN PCN.HASHFACIAL IS NOT NULL THEN 'Sim'
                    ELSE 'Não'
                END BIOMETRIA_CADASTRADA,
                CASE
                    WHEN EXISTS (
                        SELECT 1
                        FROM FPA_FERIAS FF
                        JOIN FPA_GOZOFERIAS FG ON FF.NRFERIAS = FG.NRFERIAS
                        WHERE FF.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                        AND FF.NRORG = GPE_VINCULOM.NRORG
                        AND TRUNC(SYSDATE) BETWEEN FG.DTINIGOZOFERIAS AND FG.DTFIMGOZOFERIAS
                    ) THEN 'S'
                    ELSE 'N'
                END AS ISVINCULOFERIAS
            FROM
                GPE_VINCULOM
                JOIN GPE_VINCULOH ON GPE_VINCULOH.NRORG = GPE_VINCULOM.NRORG
                                AND GPE_VINCULOH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                                AND GPE_VINCULOH.DTMESCOMPETENC = (
                    SELECT MAX(H1.DTMESCOMPETENC)
                    FROM GPE_VINCULOH H1
                    WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    AND H1.NRORG = GPE_VINCULOH.NRORG
                    AND H1.NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                )
                LEFT JOIN ESTRUTURAH ON ESTRUTURAH.NRORG = GPE_VINCULOH.NRORG
                                    AND ESTRUTURAH.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTGEREN
                                    AND ESTRUTURAH.DTMESCOMPETENC = (
                    SELECT MAX(H1.DTMESCOMPETENC)
                    FROM ESTRUTURAH H1
                    WHERE H1.NRORG = ESTRUTURAH.NRORG
                    AND H1.NRESTRUTURAM = ESTRUTURAH.NRESTRUTURAM
                    AND H1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                )
                LEFT JOIN GPE_ESCALATRABM ETM ON ETM.NRORG = GPE_VINCULOM.NRORG 
                                            AND ETM.NRESCALATRABM = GPE_VINCULOH.NRESCALATRABM
                LEFT JOIN GPE_ESCALATRABH ETH ON ETH.NRORG = GPE_VINCULOM.NRORG 
                                            AND ETH.NRESCALATRABM = ETM.NRESCALATRABM 
                                            AND ETH.DTMESCOMPETENC = (
                    SELECT MAX(ETHH.DTMESCOMPETENC)
                    FROM GPE_ESCALATRABH ETHH
                    WHERE ETHH.NRORG = ETH.NRORG
                    AND ETHH.NRESCALATRABM = ETH.NRESCALATRABM
                    AND ETHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                )
                LEFT JOIN GPE_PESSOAH PH ON PH.NRORG = GPE_VINCULOM.NRORG
                                        AND PH.NRPESSOA = GPE_VINCULOM.NRPESSOA
                                        AND PH.DTMESCOMPETENC = (
                    SELECT MAX(P1.DTMESCOMPETENC)
                    FROM GPE_PESSOAH P1
                    WHERE P1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    AND P1.NRORG = PH.NRORG
                    AND P1.NRPESSOA = PH.NRPESSOA
                )
                LEFT JOIN GPE_PESSOA PA ON PA.NRORG = GPE_VINCULOM.NRORG
                    AND PA.NRPESSOA = GPE_VINCULOM.NRPESSOA
                LEFT JOIN PARCNEGOCIO PCN ON PCN.NRORG = PA.NRORG
                    AND PCN.NRPARCNEGOCIO = PA.NRPARCNEGOCIO
                LEFT JOIN GPE_SITUFUNCH SFH ON (SFH.NRORG IS NULL OR SFH.NRORG = :P_NRORG_PADRAO OR SFH.NRORG = GPE_VINCULOM.NRORG)
                                            AND SFH.NRSITUFUNCM = GPE_VINCULOH.NRSITUFUNCM
                                            AND SFH.DTEFETIVA = (
                    SELECT MAX(SFHH.DTEFETIVA)
                    FROM GPE_SITUFUNCH SFHH
                    WHERE SFHH.DTEFETIVA <= :P_DTMESCOMPETENC
                    AND (SFHH.NRORG IS NULL OR SFHH.NRORG = :P_NRORG_PADRAO OR SFHH.NRORG = GPE_VINCULOM.NRORG)
                    AND SFHH.NRSITUFUNCM = SFH.NRSITUFUNCM
                )
                LEFT JOIN (
                    SELECT
                        NRVINCULOM,
                        LISTAGG(TO_CHAR(DTHRAJUSTADA, 'HH24:MI'), ';') WITHIN GROUP (ORDER BY DTHRAJUSTADA) HORARIOS_MARCACOES_DIA_ATUAL
                    FROM GPE_MARCACAOPONTO
                    WHERE NRORG = :P_NRORG
                    AND (
                        CASE
                            WHEN IDDIAOCORRENCIA IN ('FA', 'DA') THEN TRUNC(DTHRAJUSTADA) - 1
                            WHEN IDDIAOCORRENCIA IN ('FS', 'DS') THEN TRUNC(DTHRAJUSTADA) + 1
                            ELSE TRUNC(DTHRAJUSTADA)
                        END
                    ) = TRUNC(SYSDATE)
                GROUP BY NRVINCULOM
                ) MARCACOES_DIA ON MARCACOES_DIA.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                LEFT JOIN GPE_ALTESTABVINC ESTV ON ESTV.NRORG = GPE_VINCULOM.NRORG 
                    AND ESTV.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
                    AND ESTV.DTINIESTABVINC = (
                        SELECT MAX(DTINIESTABVINC)
                        FROM GPE_ALTESTABVINC
                        WHERE NRORG = ESTV.NRORG
                            AND NRVINCULOM = ESTV.NRVINCULOM
                            AND (
                                :P_DTMESCOMPETENC >= DTINIESTABVINC AND
                                NVL(:P_DTLIMITERESC, :P_DTMESCOMPETENC) <= DTFIMESTABVINC
                            )
                    )
                LEFT JOIN GPE_MOTIVOESTAB MTVES ON MTVES.NRMOTIVOESTAB = ESTV.NRMOTIVOESTAB
                LEFT JOIN FPA_PARAMSINDICATO PSCT ON PSCT.NRORG = GPE_VINCULOM.NRORG 
                    AND PSCT.NRESTRUTURAM = GPE_VINCULOH.NRESTRUTSIND
            WHERE
                GPE_VINCULOM.NRORG = :P_NRORG
                AND (
                        :P_NRESTRUTURAS IS NULL
                        OR (INSTR(:P_NRESTRUTURAS, ',' || ESTRUTURAH.NRESTRUTURAM || ',')) > 0 )
                AND ( GPE_VINCULOM.DTRESCISAOVINC IS NULL
                    OR GPE_VINCULOM.DTRESCISAOVINC >= TO_DATE(NVL(:P_DTLIMITERESC, :P_DTMESCOMPETENC), 'DD/MM/YYYY') )
                AND GPE_VINCULOM.NRVINCULOM <> NVL(:P_NRVINCULOM_GESTOR, -1)
            ORDER BY
                GPE_VINCULOM.NMVINCULOM
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getVinculosByPerfAcessRegPntMlt($params) {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.NMVINCULOM,
                VM.DTADMISSAOVINC,
                VM.DTRESCISAOVINC,
                VM.NRTIPOVINCULOM,
                VM.NRPESSOA,
                PH.NRCPFPESSOA,
                CASE
                    WHEN VH.NRESCALATRABM IS NOT NULL THEN
                        VM.NMVINCULOM || ' (' || ETH.NMESCALATRABH || ')'
                    ELSE
                        VM.NMVINCULOM
                END AS NMVINCULOM_NMESCALATRABH,
                CASE
                    WHEN EXISTS (
                        SELECT 1
                        FROM FPA_FERIAS FF
                        JOIN FPA_GOZOFERIAS FG ON FF.NRFERIAS = FG.NRFERIAS
                        WHERE FF.NRVINCULOM = VM.NRVINCULOM
                        AND FF.NRORG = VM.NRORG
                        AND TRUNC(SYSDATE) BETWEEN FG.DTINIGOZOFERIAS AND FG.DTFIMGOZOFERIAS
                    ) THEN 'S'
                    ELSE 'N'
                END AS ISVINCULOFERIAS,
                MARCACOES_DIA.HORARIOS_MARCACOES_DIA_ATUAL
            FROM PARCNEGOCIO PN
            JOIN GPE_PESSOA P ON P.NRORG = PN.NRORG
                AND P.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
            JOIN GPE_VINCULOM VM ON VM.NRORG = PN.NRORG
                AND VM.NRPESSOA = P.NRPESSOA
                AND VM.DTADMISSAOVINC <= TRUNC(SYSDATE)
                AND (VM.DTRESCISAOVINC >= TRUNC(SYSDATE) OR VM.DTRESCISAOVINC IS NULL)
            JOIN GPE_VINCULOH VH ON VH.NRORG = PN.NRORG
                AND VH.NRVINCULOM = VM.NRVINCULOM
                AND VH.DTMESCOMPETENC = (
                    SELECT DTMESCOMPETENC FROM (SELECT
                        VHH.DTMESCOMPETENC
                    FROM
                        GPE_VINCULOH VHH
                    WHERE
                        VHH.NRORG = VH.NRORG
                        AND VHH.NRVINCULOM = VH.NRVINCULOM
			ORDER BY VHH.DTMESCOMPETENC DESC
            ) WHERE ROWNUM = 1)
            LEFT JOIN GPE_ESCALATRABH ETH ON ETH.NRORG = VM.NRORG
                AND ETH.NRESCALATRABM = VH.NRESCALATRABM
                AND ETH.DTMESCOMPETENC = (
                    (SELECT DTMESCOMPETENC FROM 
                        (SELECT 
                            ETH1.DTMESCOMPETENC 
                         FROM 
                            GPE_ESCALATRABH ETH1 
                         WHERE 
                           ETH1.NRORG = ETH.NRORG AND
                           ETH1.NRESCALATRABM = ETH.NRESCALATRABM AND
                           ETH1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                         ORDER BY ETH1.DTMESCOMPETENC DESC) 
                  WHERE ROWNUM = 1)
                )
            LEFT JOIN GPE_PESSOAH PH ON PH.NRORG = VM.NRORG
                AND PH.NRPESSOA = VM.NRPESSOA
                AND PH.DTMESCOMPETENC = ( SELECT DTMESCOMPETENC FROM (
                SELECT
                    P1.DTMESCOMPETENC
                FROM
                    GPE_PESSOAH P1
                WHERE
                    P1.NRORG = PH.NRORG
                    AND P1.NRPESSOA = PH.NRPESSOA
                    AND P1.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
		ORDER BY P1.DTMESCOMPETENC DESC
            ) WHERE ROWNUM = 1)
            LEFT JOIN (
                SELECT
                    NRVINCULOM,
                    LISTAGG(TO_CHAR(DTHRAJUSTADA, 'HH24:MI'), ';')
                        WITHIN GROUP (ORDER BY DTHRAJUSTADA) HORARIOS_MARCACOES_DIA_ATUAL
                FROM GPE_MARCACAOPONTO
                WHERE NRORG = :P_NRORG
                    AND (
                        CASE
                            WHEN IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                                TRUNC(DTHRAJUSTADA) - 1
                            WHEN IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                                TRUNC(DTHRAJUSTADA) + 1
                            ELSE
                                TRUNC(DTHRAJUSTADA)
                        END
                    ) = TRUNC(SYSDATE)
                GROUP BY NRVINCULOM
            ) MARCACOES_DIA ON MARCACOES_DIA.NRVINCULOM = VM.NRVINCULOM
            WHERE PN.NRORG = :P_NRORG
                AND EXISTS (
                    SELECT 1
                    FROM PERFACESSPARC PA
                    LEFT JOIN GPE_PARAMORG PO ON PO.NRORG = PA.NRORG
                    WHERE PA.NRORG = PN.NRORG
                    AND PA.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
                    AND ( INSTR(',' || PO.NRPERFACESSREGPNTMLT || ',', ',' || PA.NRPERFILACESSO  || ',') > 0 )
                )
            ORDER BY NMVINCULOM
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /*
    * Função pra poder montar o novo regitro de marcação que será inserido
    */
    public function getHorarioBase(
        string $nrvinculom,
        string $nrorg,
        string $date = null
    ) : array {
        if ($date === null) {
            $date = new \DateTime();
            $date = $date->format('d/m/Y');
        }

        try {
            $dadosHorarioVinculo = $this->getDadosHorarioVinculo($nrvinculom, $nrorg, $date);
            $nrhordiam = $dadosHorarioVinculo['NRHORDIAM'];

            $paramorg = $this->getParamorg($nrorg);

            if (empty($paramorg)) {
                throw new \Exception(constant('messagesMarcacaoRepository')['ERRO_PARAMORG_NAO_ENCONTRADO']);
            }

            $toleranciaAnterior  = DateUtil::getHoraFloatDeString($paramorg['HRTEMPOTOLERANCIAANT']);
            $toleranciaPosterior = DateUtil::getHoraFloatDeString($paramorg['HRTEMPOTOLERANCIAPOS']);


            $sql = <<<SQL

                SELECT
                    --TO_CHAR(TO_DATE(HRENTRADA, 'HH24MI') - 0.003472, 'HH24:MI') AS HRENTRADA_NOVA,
                    SUBSTR(SUBSTR(HRENTRADA, 1, LENGTH(TRIM(HRENTRADA)) -2) || ':' || SUBSTR(HRENTRADA, -2), 1, 10) AS HRENTRADA,
                    SUBSTR(SUBSTR(HRPROJETARINTERVALO, 1, LENGTH(TRIM(HRPROJETARINTERVALO)) -2) || ':' || SUBSTR(HRPROJETARINTERVALO, -2), 1, 10) AS HRPROJETARINTERVALO,
                    (SELECT SUBSTR(SUBSTR(HRSAIDA, 1, LENGTH(TRIM(HRSAIDA)) -2) || ':' || SUBSTR(HRSAIDA, -2), 1, 10) AS HRSAIDA
                        FROM GPE_INTERVALOSHORARIO IH
                    WHERE IH.NRORG     = :P_NRORG
                        AND IH.NRHORDIAM = :P_NRHORDIAM
                        AND IH.NRSEQINTERVHORARIO = (SELECT MAX(NRSEQINTERVHORARIO)
                                                            FROM GPE_INTERVALOSHORARIO
                                                        WHERE NRORG     = IH.NRORG
                                                            AND NRHORDIAM = IH.NRHORDIAM)) HRSAIDA
                FROM GPE_INTERVALOSHORARIO IH

                WHERE IH.NRORG     = :P_NRORG
                    AND IH.NRHORDIAM = :P_NRHORDIAM
                    AND IH.NRSEQINTERVHORARIO = (SELECT MIN(NRSEQINTERVHORARIO)
                                                    FROM GPE_INTERVALOSHORARIO
                                                    WHERE NRORG     = IH.NRORG
                                                    AND NRHORDIAM = IH.NRHORDIAM)

SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"     => $nrorg,
                    "P_NRHORDIAM" => $nrhordiam
                )
            );

            $horariobase = $query->fetchAll();

            if (count($horariobase) > 0) {
                $horariobase[0]["HRENTRADA_NOVA"] = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRENTRADA"]) - $toleranciaAnterior, true);
                $horariobase[0]["HRSAIDA_NOVA"]   = DateUtil::getHoraStringDeFloat(DateUtil::getHoraFloatDeString($horariobase[0]["HRSAIDA"]) + $toleranciaPosterior, true);

                return $horariobase[0];
            } else {
                throw new \Exception("Horário base não encontrado.", 1);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereMarcacoesPontoManual(
        string  $nrvinculom,
        string  $dthrmarcacaoponto,
        string  $dthrajustada,
        string  $idenquadrahorario,
        string  $idmarcdesprezada,
        string  $iddiaocorrencia,
                $dsjustificativa,
        string  $nrorginclusao,
        int     $nrcoletor,
        string  $idreferintervalo,
                $cdcnpjestrut,
                $nmrazsocestrut,
                $cdcaepfestrut,
                $cdceiestrut,
                $cdticketaut,
                $dslatitude,
                $dslongitude,
                $dspathfoto,
        string  $idativo,
        ?string $cdoperinclusao,
        string  $nrorg,
                $nrMarcacao = null
    ) : string {

        $nrmarcacaoponto =
            $nrMarcacao ??
            NovoCodigo::geraCodigo($nrorg, 'GPE_MARCACAOPONTO', 1, 22, false);

        $dtinclusao = $dthrmarcacaoponto;

        $sql = <<<SQL
            INSERT INTO GPE_MARCACAOPONTO (
                NRMARCACAOPONTO,
                NRVINCULOM,
                DTHRMARCACAOPONTO,
                DTHRAJUSTADA,
                IDENQUADRAHORARIO,
                IDMARCDESPREZADA,
                IDDIAOCORRENCIA,
                DSJUSTIFICATIVA,
                NRORGINCLUSAO,
                NRCOLETOR,
                IDREFERINTERVALO,
                CDCNPJESTRUT,
                NMRAZSOCESTRUT,
                CDCAEPFESTRUT,
                CDCEIESTRUT,
                CDTICKETAUT,
                DSLATITUDE,
                DSLONGITUDE,
                DSPATHFOTO,
                IDATIVO,
                DTINCLUSAO,
                CDOPERINCLUSAO,
                NRORG
            ) VALUES (
                :P_NRMARCACAOPONTO,
                :P_NRVINCULOM,
                TO_DATE(:P_DTHRMARCACAOPONTO, 'DD/MM/YYYY HH24:MI:SS'),
                TO_DATE(:P_DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS'),
                :P_IDENQUADRAHORARIO,
                :P_IDMARCDESPREZADA,
                :P_IDDIAOCORRENCIA,
                :P_DSJUSTIFICATIVA,
                :P_NRORGINCLUSAO,
                :P_NRCOLETOR,
                :P_IDREFERINTERVALO,
                :P_CDCNPJESTRUT,
                :P_NMRAZSOCESTRUT,
                :P_CDCAEPFESTRUT,
                :P_CDCEIESTRUT,
                :P_CDTICKETAUT,
                :P_DSLATITUDE,
                :P_DSLONGITUDE,
                :P_DSPATHFOTO,
                :P_IDATIVO,
                SYSDATE,
                :P_CDOPERINCLUSAO,
                :P_NRORG
            )
SQL;

        $query = $this->connection->prepare($sql);

        $result = $query->execute(
            array(
                "P_NRMARCACAOPONTO"     => $nrmarcacaoponto,
                "P_NRVINCULOM"          => $nrvinculom,
                "P_DTHRMARCACAOPONTO"   => $dthrmarcacaoponto,
                "P_DTHRAJUSTADA"        => $dthrajustada,
                "P_IDENQUADRAHORARIO"   => $idenquadrahorario,
                "P_IDMARCDESPREZADA"    => $idmarcdesprezada,
                "P_IDDIAOCORRENCIA"     => $iddiaocorrencia,
                "P_DSJUSTIFICATIVA"     => $dsjustificativa,
                "P_NRORGINCLUSAO"       => $nrorginclusao,
                "P_NRCOLETOR"           => $nrcoletor,
                "P_IDREFERINTERVALO"    => $idreferintervalo,
                "P_CDCNPJESTRUT"        => $cdcnpjestrut,
                "P_NMRAZSOCESTRUT"      => $nmrazsocestrut,
                "P_CDCAEPFESTRUT"       => $cdcaepfestrut,
                "P_CDCEIESTRUT"         => $cdceiestrut,
                "P_CDTICKETAUT"         => $cdticketaut,
                "P_DSLATITUDE"          => $dslatitude,
                "P_DSLONGITUDE"         => $dslongitude,
                "P_DSPATHFOTO"          => $dspathfoto,
                "P_IDATIVO"             => $idativo,
                "P_CDOPERINCLUSAO"      => $cdoperinclusao,
                "P_NRORG"               => $nrorg
            )
        );

        if ($result) {
            return $nrmarcacaoponto;
        } else {
            return false;
        }
    }


    // Dia apuração é importante pois o horário retornado corresponde a jornada de trabalho do dia específico
    public function getDadosHorarioVinculo(
        string $nrvinculom,
        string $nrorg,
        string $dtapuracaoponto = null
    )  {

        try {
            if ($dtapuracaoponto === null) {
                $dtapuracaoponto = new \DateTime();
                $dtapuracaoponto = $dtapuracaoponto->format('d/m/Y');
            }

            $vinculom = $this->getVinculom($nrvinculom, $nrorg);

            if (empty($vinculom)) {
                throw new \Exception(constant('messagesMarcacaoRepository')['ERRO_VINCULO_NAO_ENCONTRADO']);
            }

            $nrtipovinculom = $vinculom['NRTIPOVINCULOM'];
            $nrvinculoempreg = $vinculom['NRVINCULOEMPREG'];

            $whereHrprojetarintervalo = "";
            if ($nrtipovinculom == self::TIPO_VINC_CLT && $nrvinculoempreg != self::VINC_EMPREG_APRENDIZ) {
                $whereHrprojetarintervalo = <<<SQL
                    (IH.HRPROJETARINTERVALO IS NOT NULL OR IDTIPOHORARIO IN ('F', 'C')) AND
SQL;
            }

            $sql = <<<SQL
                SELECT
                    HT.NRHORDIAM,
                    HD.IDTIPOHORARIO,
                    IH.NRINTERVALOHORARIO,
                    NVL(IV.HRPROJETARINTERVALO, '0000') HRPROJETARINTERVALO,
                    HH.NMHORDIAH,
                    AE.NRESCALATRABM,
                    EH.QTHRESCALATRABH AS QTJORNADADIARIA
                FROM
                    (
                        SELECT
                            NRORG,
                            MAX(NRTURNO) NRTURNO,
                            NRESCALATRABM
                        FROM
                            GPE_ALTEESCALA
                        WHERE
                            NRORG = :P_NRORG
                            AND
                            NRVINCULOM = :P_NRVINCULOM
                            AND
                            DTINIESCALA = (
                                SELECT MAX(DTINIESCALA)
                                FROM GPE_ALTEESCALA ALT
                                WHERE
                                    ALT.NRORG = GPE_ALTEESCALA.NRORG
                                    AND
                                    ALT.NRVINCULOM = GPE_ALTEESCALA.NRVINCULOM
                                    AND
                                    ALT.DTINIESCALA <= TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY'))
                        GROUP BY
                            NRORG,
                            NRESCALATRABM
                    ) AE,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - DTDATABASE AS NRDIASENTRE
                        FROM GPE_TURNO
                        WHERE
                            NRORG = :P_NRORG
                    ) TU,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            NRHORDIAM,
                            ROW_NUMBER() OVER (PARTITION BY NRORG, NRTURNO ORDER BY NRSEQHORARIO) AS NRSEQHR
                        FROM
                            GPE_HORARIOTURNO
                        WHERE
                            NRORG = :P_NRORG
                        ORDER BY
                            NRORG,
                            NRTURNO,
                            NRSEQHORARIO
                    ) HT,
                    (
                        SELECT
                            NRORG,
                            NRTURNO,
                            COUNT(*) AS QTHORARIOS
                        FROM
                            GPE_HORARIOTURNO
                        WHERE
                            NRORG = :P_NRORG
                        GROUP BY
                            NRORG,
                            NRTURNO
                    ) QH,
                    (
                        SELECT
                            NRORG,
                            NRHORDIAM,
                            MAX(DTMESCOMPETENC) AS DTMESCOMPETENC,
                            NMHORDIAH
                        FROM GPE_HORDIAH
                        WHERE
                            NRORG = :P_NRORG
                            AND
                            DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY')
                        GROUP BY
                            NRORG,
                            NRHORDIAM,
                            NMHORDIAH
                    ) HH,
                    (
                        SELECT
                            NRORG,
                            NRHORDIAM,
                            MIN(NRSEQINTERVHORARIO) NRSEQINTERVHORARIO,
                            HRPROJETARINTERVALO
                        FROM
                            GPE_INTERVALOSHORARIO
                        WHERE
                            NRORG = :P_NRORG
                        GROUP BY
                            NRORG,
                            NRHORDIAM,
                            HRPROJETARINTERVALO
                    ) IV,
                    GPE_INTERVALOSHORARIO IH,
                    GPE_HORDIAH HD,
                    GPE_ESCALATRABH EH
                WHERE
                    AE.NRORG = TU.NRORG AND
                    AE.NRTURNO = TU.NRTURNO AND
                    AE.NRORG = HT.NRORG AND
                    AE.NRTURNO = HT.NRTURNO AND
                    HT.NRORG = HH.NRORG AND
                    HT.NRHORDIAM = HH.NRHORDIAM AND
                    HD.NRORG = HH.NRORG AND
                    HD.NRHORDIAM = HH.NRHORDIAM AND
                    HD.DTMESCOMPETENC = HH.DTMESCOMPETENC AND
                    IV.NRORG(+) = HH.NRORG AND
                    IV.NRHORDIAM(+) = HH.NRHORDIAM AND
                    IH.NRORG(+) = IV.NRORG AND
                    IH.NRHORDIAM(+) = IV.NRHORDIAM AND
                    IH.NRSEQINTERVHORARIO(+) = IV.NRSEQINTERVHORARIO AND
                    AE.NRORG = QH.NRORG AND
                    AE.NRTURNO = QH.NRTURNO AND
                    HT.NRSEQHR - 1 = MOD(TU.NRDIASENTRE, QH.QTHORARIOS) AND
                    $whereHrprojetarintervalo
                    EH.NRESCALATRABM = AE.NRESCALATRABM AND
                    EH.NRORG = AE.NRORG AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESCALATRABM = AE.NRESCALATRABM AND
                            DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENCIA, 'DD/MM/YYYY'))
                ORDER BY IH.NRSEQINTERVHORARIO
SQL;

            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"            => $nrorg,
                    "P_NRVINCULOM"       => $nrvinculom,
                    "P_DTMESCOMPETENCIA" => DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO),
                    "P_DTAPURACAOPONTO"  => $dtapuracaoponto
                )
            );

            $dadosHorario = $query->fetchAll();

            if (isset($dadosHorario[0])) {
                return $dadosHorario[0];
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getVinculom(
        $nrvinculom,
        $nrorg)
    {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    GPE_VINCULOM
                WHERE
                    NRORG = :P_NRORG
                    AND NRVINCULOM = :P_NRVINCULOM
SQL;

            $params = array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom
            );

            $result = $this->connection->fetchAll($sql, $params);

            return !empty($result) ? $result[0] : $result;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParamorg($nrorg) {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    GPE_PARAMORG
                WHERE
                    NRORG = :P_NRORG
SQL;

            $params = array(
                'P_NRORG' => $nrorg
            );

            $result = $this->connection->fetchAll($sql, $params);

            return !empty($result) ? $result[0] : $result;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParesMarcacoesDia(array $params) : array {
        $sql = <<<SQL
            SELECT
              ROWNUM,
              NRMARCACAOPONTO,
              MARC.NRVINCULOM,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS DTHRAJUSTADA,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
              DTHRAJUSTADA AS DTHRORDER,
              IDMARCDESPREZADA,
              IDDIAOCORRENCIA,
              DSJUSTIFICATIVA,
              COL.NRCOLETOR,
              COL.DSCOLETOR,
              IDREFERINTERVALO
            FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
               ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND MARC.NRVINCULOM = :P_NRVINCULO
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA = 'DC')
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                   OR (IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                   OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                   OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY DTHRORDER, NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $paresMarcacoes = $std->fetchAll();

        return $paresMarcacoes;
    }
    
    public function getMarcacaoByNrmarcacaoponto(
        int $nrorg,
        int $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            SELECT
              MP.*,
              TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY') DIA,
              TO_CHAR(MP.DTHRMARCACAOPONTO, 'HH24:MI') HORA_MINUTO
            FROM GPE_MARCACAOPONTO MP
            WHERE MP.NRORG = :P_NRORG
                AND MP.NRMARCACAOPONTO = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }

    public function removeParMarcacao(array $params) : int {
        $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
            WHERE NRORG             = :P_NRORG
              AND NRVINCULOM        = :P_NRVINCULOM
              AND NRMARCACAOPONTO   IN (:P_NRMARCACAOENTRADA, :P_NRMARCACAOSAIDA)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacaoesRemovidas = $std->rowCount();

        return $marcacaoesRemovidas;
    }
    
    public function removeMarcacao(array $params) : int {
        $sql = <<<SQL
            DELETE FROM GPE_MARCACAOPONTO
            WHERE NRORG             = :P_NRORG
              AND NRVINCULOM        = :P_NRVINCULOM
              AND NRMARCACAOPONTO   = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacaoesRemovidas = $std->rowCount();

        return $marcacaoesRemovidas;
    }

    public function saveMarcacao(array $params) : bool {
        $params['P_NRMARCACAO'] = isset($params['P_NRMARCACAO']) && isset($params['P_NRORG']) ? $params['P_NRMARCACAO'] : NovoCodigo::geraCodigo($params['P_NRORG'], 'GPE_MARCACAOPONTO', 1, 22, false);
        
        $sql = <<<SQL
            INSERT INTO GPE_MARCACAOPONTO (
                      NRMARCACAOPONTO,
                      NRVINCULOM,
                      DTHRMARCACAOPONTO,
                      DTHRAJUSTADA,
                      IDENQUADRAHORARIO,
                      IDMARCDESPREZADA,
                      IDDIAOCORRENCIA,
                      DSJUSTIFICATIVA,
                      NRORGINCLUSAO,
                      NRCOLETOR,
                      IDREFERINTERVALO,
                      IDATIVO,
                      DTINCLUSAO,
                      CDOPERINCLUSAO,
                      NRORG,
                      IDEXIGEAPROVACAO
                  ) VALUES (
                      :P_NRMARCACAO,
                      :P_NRVINCULOM,
                      TO_DATE(:P_DTHRMARCACAO, 'DD/MM/YYYY HH24:MI:SS'),
                      TO_DATE(:P_DTHRMARCACAO, 'DD/MM/YYYY HH24:MI:SS'),
                      'N',
                      'N',
                      :P_IDDIAOCORRENCIA,
                      '',
                      '',
                      :P_NRCOLETOR,
                      'N',
                      'S',
                      SYSDATE,
                      1,
                      :P_NRORG,
                      :P_IDEXIGEAPROVACAO
                   )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function getMarcacoesDia(array $params): array {
        $sql = <<<SQL
            SELECT MP.*,
                   TO_CHAR(DTHRAJUSTADA, 'HH24:MI') HORA_MINUTO
            FROM GPE_MARCACAOPONTO MP
            WHERE NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULO
              AND ((IDDIAOCORRENCIA IN ('FA', 'DA') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') + 1)
                OR (IDDIAOCORRENCIA IN ('FS', 'DS') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY') - 1)
                OR (IDDIAOCORRENCIA IN ('FC', 'DC') AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DATE,'DD/MM/YYYY')))
              AND NRCOLETOR NOT IN (
                SELECT NRCOLETOR
                FROM GPE_PARAMORG
                UNPIVOT (NRCOLETOR FOR NRCOLETOR_TIPO IN (NRCOLETORMARCAUTOM , NRCOLETORMARCGERSIST, NRCOLETORMARCVIRADIA, NRCOLETORMARCPROJETADA))
                WHERE NRORG = :P_NRORG
              )
            ORDER BY DTHRAJUSTADA ASC, NRMARCACAOPONTO ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoes = $std->fetchAll();
        
        return $marcacoes;
    }
    
    public function getPrimeiraAusenciaMarcacaoDia(
        string $nrorg,
        string $nrvinculom,
        string $date
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                (
                    SELECT  
                        ROWNUM RN,
                        MARCACAOPONTO.* 
                    FROM (
                        SELECT
                            *
                        FROM
                            GPE_MARCACAOPONTO MP
                        WHERE
                            NRORG = :P_NRORG
                            AND NRVINCULOM = :P_NRVINCULOM
                            AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                            AND NRCOLETOR <> 21 /* BALIZA */
                            
                        ORDER BY
                            DTHRMARCACAOPONTO
                    ) MARCACAOPONTO
                    
                    WHERE
                        (
                            SELECT
                                COUNT(*) ROWCOUNT
                            FROM
                            GPE_MARCACAOPONTO MP
                            WHERE
                                NRORG = :P_NRORG
                                AND NRVINCULOM = :P_NRVINCULOM
                                AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                                AND NRCOLETOR <> 21 /* BALIZA */
                        ) > 2 /* GARANTE QUE TENHA PELO MENOS UM INTERVALO */
                )
                
            WHERE
                RN = 2 /* PRIMERIA AUSÊNCIA */
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'               => $nrorg,
                'P_NRVINCULOM'          => $nrvinculom,
                'P_DTHRMARCACAOPONTO'   => $date
            )    
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }
    
    public function getMarcacaoDiaByOrdem(
        string  $nrorg,
        string  $nrvinculom,
        string  $date,
        int     $posicaoMarcacaoDia = 1,
        string  $ordem              = Utils::ORDEM_ASC
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                (
                    SELECT  
                        ROWNUM RN,
                        MARCACAOPONTO.* 
                    FROM (
                        SELECT
                            *
                        FROM
                            GPE_MARCACAOPONTO MP
                        WHERE
                            NRORG = :P_NRORG
                            AND NRVINCULOM = :P_NRVINCULOM
                            AND 
                                (
                                 (TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY')          = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FC')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO + 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FA')
                                 OR (TO_CHAR(DTHRMARCACAOPONTO - 1, 'DD/MM/YYYY')   = :P_DTHRMARCACAOPONTO AND IDDIAOCORRENCIA = 'FS')
                                )
                            AND NRCOLETOR <> 21 /* BALIZA */
                            
                        ORDER BY
                            DTHRMARCACAOPONTO $ordem
                    ) MARCACAOPONTO
                )
                
            WHERE
                RN = :P_POSICAO_MARCACAO_DIA /* SELECIONA A MARCAÇÃO SEQUENCIAL */
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'                   => $nrorg,
                'P_NRVINCULOM'              => $nrvinculom,
                'P_DTHRMARCACAOPONTO'       => $date,
                'P_POSICAO_MARCACAO_DIA'    => $posicaoMarcacaoDia,
            )    
        );

        $marcacoes = $std->fetch();

        return $marcacoes;
    }
    
    public function getMarcacaoDiaEntrada(
        string  $nrorg,
        string  $nrvinculom,
        string  $date
    ) {
        
        return $this->getMarcacaoDiaByOrdem(
            $nrorg,
            $nrvinculom,
            $date,
            MarcacaoService::POSICAO_MARCACAO_ENTRADA,
            MarcacaoService::ORDEM_MARCACAO_ENTRADA
        );
    }
    
    public function getMarcacaoDiaSaida(
        string  $nrorg,
        string  $nrvinculom,
        string  $date
    ) {
        
        return $this->getMarcacaoDiaByOrdem(
            $nrorg,
            $nrvinculom,
            $date,
            MarcacaoService::POSICAO_MARCACAO_SAIDA,
            MarcacaoService::ORDEM_MARCACAO_SAIDA
        );
    }
    
    public function getMarcacaoByDataMarcacao(
        $nrorg,
        $nrvinculom,
        $horaMarcacao,
        $diaMarcacao,
        bool $all = false,
        $idMarcacao = 'FC'
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_MARCACAOPONTO MP
            WHERE
                MP.NRORG                                        = :P_NRORG
                AND MP.NRVINCULOM                               = :P_NRVINCULOM
                AND TO_CHAR(mp.dthrmarcacaoponto, 'HH24:MI')    = :P_HORAMARCACAO
                AND (
                    (
                        TO_CHAR(mp.dthrmarcacaoponto, 'DD/MM/YYYY') = :P_DIAMARCACAO
                        AND :P_IDMARCACAO IN ('FC', 'DC')
                        AND mp.IDDIAOCORRENCIA IN ('FC', 'DC')
                    ) OR
                    (
                        TO_CHAR(mp.dthrmarcacaoponto + 1, 'DD/MM/YYYY') = :P_DIAMARCACAO
                        AND :P_IDMARCACAO IN ('FS', 'DS')
                        AND mp.IDDIAOCORRENCIA IN ('FS', 'DS')
                    ) OR
                    (
                        TO_CHAR(mp.dthrmarcacaoponto - 1, 'DD/MM/YYYY') = :P_DIAMARCACAO
                        AND :P_IDMARCACAO IN ('FA', 'DA')
                        AND mp.IDDIAOCORRENCIA IN ('FA', 'DA')
                    )
                )
                AND MP.NRCOLETOR <> 21
        
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
                'P_HORAMARCACAO'    => $horaMarcacao,
                'P_DIAMARCACAO'     => $diaMarcacao,
                'P_IDMARCACAO'      => $idMarcacao,
            )
        );

        $marcacao = $all ?
            $std->fetchAll() :
            $std->fetch();
        
        return $marcacao;
    }
    
    public function getMarcacaoByDiaMarcacao(
        $nrorg,
        $nrvinculom,
        $diaMarcacao
    ) {
        
        if (!is_array($diaMarcacao)) {
            $paramDiaMarcacao = array($diaMarcacao);
        } else {
            $paramDiaMarcacao = $diaMarcacao;
        }
        
        $paramDiaMarcacao = '\'' . implode('\', \'', $paramDiaMarcacao) . '\'';
        
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_MARCACAOPONTO MP
            WHERE
                MP.NRORG                                        = :P_NRORG
                AND MP.NRVINCULOM                               = :P_NRVINCULOM
                AND TO_CHAR(MP.DTHRMARCACAOPONTO, 'DD/MM/YYYY') IN ($paramDiaMarcacao)
                AND MP.NRCOLETOR                                <> 21
        
SQL;

        $std = $this->connection->prepare($sql);
        
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRVINCULOM'      => $nrvinculom,
            )    
        );

        $marcacao = $std->fetchAll();
        
        return $marcacao;
    }
    
    public function limpaIdFixaMarcacaoDia(
        $nrOrg,
        $nrVinculom,
        $dia,
        $idFixaEntradaSaida
    ) {
        
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND IDFIXAENTSAID            = :P_IDFIXAENTSAID
SQL;

        $std = $this->connection->prepare($sql);

        $result = $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTAPURACAOPONTO" => $dia,
                "P_IDFIXAENTSAID"   => $idFixaEntradaSaida
            )
        );
        
        return $result;
    }
    
    public function limpaIdFixaMarcacaoDiaEntrada(
        $nrOrg,
        $nrVinculom,
        $dia
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_ENTRADA;
        
        return $this->limpaIdFixaMarcacaoDia(
            $nrOrg,
            $nrVinculom,
            $dia,
            $idFixaEntradaSaida
        );
    }
    
    public function limpaIdFixaMarcacaoDiaSaida(
        $nrOrg,
        $nrVinculom,
        $dia
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_SAIDA;
        
        return $this->limpaIdFixaMarcacaoDia(
            $nrOrg,
            $nrVinculom,
            $dia,
            $idFixaEntradaSaida
        );
    }
    
    public function setIdFixaMarcacao(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto,
        $idFixaEntSaid
    ) {
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = :P_IDFIXAENTSAID
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DTAPURACAOPONTO, 'DD/MM/YYYY')
              AND NRMARCACAOPONTO          = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);

        $result = $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTAPURACAOPONTO" => $dtApuracaoPonto,
                "P_NRMARCACAOPONTO" => $nrMarcacaoPonto,
                "P_IDFIXAENTSAID"   => $idFixaEntSaid
            )
        );
        
        return $result;
    }
    
    public function setIdFixaMarcacaoSaida(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_SAIDA;
        
        return $this->setIdFixaMarcacao(
            $nrOrg,
            $nrVinculom,
            $dtApuracaoPonto,
            $nrMarcacaoPonto,
            $idFixaEntradaSaida
        );
    }
    
    public function setIdFixaMarcacaoEntrada(
        $nrOrg,
        $nrVinculom,
        $dtApuracaoPonto,
        $nrMarcacaoPonto
    ) {
        $idFixaEntradaSaida = MarcacaoService::IDFIXAENTSAID_ENTRADA;
        
        return $this->setIdFixaMarcacao(
            $nrOrg,
            $nrVinculom,
            $dtApuracaoPonto,
            $nrMarcacaoPonto,
            $idFixaEntradaSaida
        );
    }
    
    public function getMarcacoesPontoDiaOpcoesFixarMarcacao(
        array $params
    ): array {

        $sql = <<<SQL
            SELECT
              NRMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY HH24:MI:SS') AS NRMARCACAOPONTO_LABEL,
              TO_CHAR(DTHRAJUSTADA, 'DD/MM/YYYY') AS DTMARCACAOPONTO,
              TO_CHAR(DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO
             FROM GPE_MARCACAOPONTO MARC
             JOIN GPE_COLETOR COL
              ON COL.NRCOLETOR = MARC.NRCOLETOR
            WHERE MARC.NRORG = :P_NRORG
              AND MARC.NRCOLETOR != 21
              AND MARC.NRCOLETOR != 25
              AND NRVINCULOM = :P_NRVINCULOM
              AND (((IDDIAOCORRENCIA IS NULL OR IDDIAOCORRENCIA IN ('DA', 'DC', 'DS'))
                     AND TO_DATE(TO_CHAR(DTHRAJUSTADA,'DD/MM/YYYY')) = :P_DTAPURACAOPONTO)
                  OR (IDDIAOCORRENCIA = 'FA' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') + 1)
                  OR (IDDIAOCORRENCIA = 'FS' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY') - 1)
                  OR (IDDIAOCORRENCIA = 'FC' AND TRUNC(DTHRAJUSTADA) = TO_DATE(:P_DTAPURACAOPONTO,'DD/MM/YYYY')))
            ORDER BY NRMARCACAOPONTO_LABEL, NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoesPonto = $std->fetchAll();

        return $marcacoesPonto;
    }
    
    public function getParMarcacaoAlmoco(
        $nrOrg,
        $nrVinculom,
        $dtMarcacaoPonto    
    ) {

        $sql = <<<SQL
            SELECT
                MARCACAO_ALMOCO.DTHRMARCACAOPONTO       AS DTHRMARCACAOPONTO_ALMOCO,
                MARCACAO_ALMOCO.NRMARCACAOPONTO         AS NRMARCACAOPONTO_ALMOCO,
                MARCACAO_ALMOCO.IDDIAOCORRENCIA         AS IDDIAOCORRENCIA_ALMOCO,
                MARCACAO_POS_ALMOCO.DTHRMARCACAOPONTO   AS DTHRMARCACAOPONTO_POS_ALMOCO,
                MARCACAO_POS_ALMOCO.NRMARCACAOPONTO     AS NRMARCACAOPONTO_POS_ALMOCO,
                MARCACAO_POS_ALMOCO.IDDIAOCORRENCIA     AS IDDIAOCORRENCIA_POS_ALMOCO
            FROM
                (
                    SELECT
                        MP.*
                    FROM
                        PTO_PROGOCORRENCIA   PO
                        JOIN GPE_MARCACAOPONTO    MP ON PO.NRORG = MP.NRORG
                                                     AND PO.NRMARCACAOPONTO = MP.NRMARCACAOPONTO
                    WHERE
                        PO.NRORG = :P_NRORG
                        AND PO.NRVINCULOM = :P_NRVINCULOM
                        AND TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') = :P_DTMARCACAOPONTO
                        AND PO.NRTIPOPROGOCORRENCIA IN (20, 26) /* ALMOÇO */
                ) MARCACAO_ALMOCO
            JOIN (
                    SELECT
                        *
                    FROM
                        (
                            SELECT
                                *
                            FROM
                                GPE_MARCACAOPONTO MP2
                    WHERE
                                NRORG = :P_NRORG
                                AND NRVINCULOM = :P_NRVINCULOM
                                AND DTHRMARCACAOPONTO > (
                                    SELECT
                                        DTHRMARCACAOPONTO
                                    FROM
                                        (
                                            SELECT
                                                MP.*
                                            FROM
                                                PTO_PROGOCORRENCIA   PO
                                                JOIN GPE_MARCACAOPONTO    MP ON PO.NRORG = MP.NRORG
                                                                             AND PO.NRMARCACAOPONTO = MP.NRMARCACAOPONTO
                                    WHERE
                                            PO.NRORG = :P_NRORG
                                            AND PO.NRVINCULOM = :P_NRVINCULOM
                                            AND TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') = :P_DTMARCACAOPONTO
                                            AND PO.NRTIPOPROGOCORRENCIA IN (20, 26)
                                            AND ROWNUM = 1
                                    )
                                )
                                /* MARCACOES VALIDAS APENAS */
                                AND MP2.NRCOLETOR NOT IN (
                                    SELECT NRCOLETOR
                                    FROM GPE_PARAMORG
                                    UNPIVOT (NRCOLETOR FOR NRCOLETOR_TIPO IN (NRCOLETORMARCAUTOM , NRCOLETORMARCGERSIST, NRCOLETORMARCVIRADIA, NRCOLETORMARCPROJETADA))
                                    WHERE NRORG = :P_NRORG
                                )
                            ORDER BY
                                MP2.DTHRMARCACAOPONTO
                        )
                    WHERE
                        ROWNUM = 1
                ) MARCACAO_POS_ALMOCO ON MARCACAO_POS_ALMOCO.NRORG = MARCACAO_ALMOCO.NRORG
                                         AND MARCACAO_POS_ALMOCO.NRVINCULOM = MARCACAO_ALMOCO.NRVINCULOM
SQL;

        $std = $this->connection->prepare($sql);

        $std->execute(
            array(
                "P_NRORG"           => $nrOrg,
                "P_NRVINCULOM"      => $nrVinculom,
                "P_DTMARCACAOPONTO" => $dtMarcacaoPonto
            )
        );

        $marcacoesPonto = $std->fetch();

        return $marcacoesPonto;

    }

    public function reverterEntradaSaida(
        array $params
    ) {
        $sql = <<<SQL
            UPDATE
              GPE_MARCACAOPONTO
            SET
              IDFIXAENTSAID = ''
            WHERE NRORG                    = :P_NRORG
              AND NRVINCULOM               = :P_NRVINCULOM
              AND TRUNC(DTHRMARCACAOPONTO) = TO_DATE(:P_DATE, 'DD/MM/YYYY')
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function removeMarcacoesDia(
        array $params
    ) {
        $sql = <<<SQL
            DELETE FROM
                GPE_MARCACAOPONTO
            WHERE
                NRORG = :P_NRORG AND
                NRVINCULOM = :P_NRVINCULO AND
                TO_CHAR(DTHRMARCACAOPONTO, 'DD/MM/YYYY') = :P_DATE
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getDetalheAcompanhamento(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                TO_CHAR(MARC.DTHRAJUSTADA, 'YYYY/MM/DD') DATE_ORDER,
                TO_CHAR(MARC.DTHRAJUSTADA, 'HH24:MI') AS HORA_MINUTO,
                TO_CHAR(MARC.DTHRAJUSTADA, 'DD/MM/YYYY') AS DTAPURACAOSITUFREQ,
                MARC.NRMARCACAOPONTO,
                MARC.IDDIAOCORRENCIA,
                MARC.DSLATITUDE,
                MARC.DSLONGITUDE,
                MARC.NRCOLETOR,
                (
                    CASE
                        WHEN NVL(MARC.IDEXIGEAPROVACAO, 'N') <> 'N' THEN NVL(PROG_APROV.IDPROGOCORCONFIRMADA, 'P')
                        ELSE 'S'
                    END
                ) ID_STATUS_MARC,
                (
                    CASE
                        WHEN MARC.NRCOLETOR IN (
                            PO.NRCOLETORGESTOR,
                            PO.NRCOLETORMARCAUTOM,
                            PO.NRCOLETORMARCGERSIST,
                            PO.NRCOLETORMARCVIRADIA,
                            PO.NRCOLETORMARCPROJETADA
                        ) THEN 'S'
                        ELSE 'N'
                    END
                ) ID_EDITABLE_PG
            FROM
                GPE_MARCACAOPONTO MARC
            LEFT JOIN GPE_PARAMORG PO ON PO.NRORG = MARC.NRORG
            LEFT JOIN PTO_PROGOCORRENCIA PROG_APROV ON PROG_APROV.NRORG = MARC.NRORG
                AND PROG_APROV.NRVINCULOM = MARC.NRVINCULOM
                AND PROG_APROV.NRMARCACAOPONTO = MARC.NRMARCACAOPONTO
                AND PROG_APROV.IDATIVO = 'S'
                AND PROG_APROV.NRTIPOPROGOCORRENCIA = 30
            WHERE
                MARC.NRORG = :P_NRORG
                AND MARC.NRCOLETOR          != 21
                AND MARC.NRCOLETOR          != 25
                AND MARC.NRVINCULOM         = :P_NRVINCULOM
                AND INSTR(
                        :P_DATES,
                        ',' ||
                        (
                            CASE
                                WHEN MARC.IDDIAOCORRENCIA IN ('DS', 'FS') THEN TO_CHAR(MARC.DTHRAJUSTADA + 1, 'DD/MM/YYYY')
                                WHEN MARC.IDDIAOCORRENCIA IN ('DA', 'FA') THEN TO_CHAR(MARC.DTHRAJUSTADA - 1, 'DD/MM/YYYY')
                                ELSE TO_CHAR(MARC.DTHRAJUSTADA, 'DD/MM/YYYY')
                            END
                        ) ||
                        ','
                    ) > 0
            ORDER BY MARC.DTHRAJUSTADA ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $marcacoes = $std->fetchAll();

        return $marcacoes;
    }
    
    /**
     * Busca pelos emails e conteudo a ser enviado alertando sobre ausencias de marcacoes na ultima semana
     * 
     * @param array  | null $nrVinculomList
     * @param array  | null $nrEstruturamLegal
     * @param array  | null $nrEstruturamGeren
     * @param array  | null $nrTipoVinculomList
     * @param array  | null $nrColetorList
     * @param string | null $dtInicioVerificacao
     * @param string | null $dtFinalVerificacao
     * @return array
     */
    public function getAlertaRegistrosFaltantesUltimaSemana(
        ?array $nrVinculomList = [], ?array $nrEstruturamLegal = [], ?array $nrEstruturamGeren = [], ?array $nrTipoVinculomList = [], ?array $nrColetorList = [],
        ?string $dtInicioVerificacao = null, ?string $dtFinalVerificacao = null
    ): array {
        $sql = <<<SQL
            /* Baseado na consulta do datasource getMarcacaoPontoOcorrencias */
            WITH VINCULOS AS (
                SELECT
                    VM.NRORG,
                    VM.NRVINCULOM,
                    VM.NMVINCULOM,
                    VM.NRTIPOVINCULOM,
                    VH.NRESTRUTGEREN,
                    VH.NRESTRUTLEGAL,
                    NVL(OP.DSEMAILOPER, CP.CDCOMUNICAPARC) EMAIL,
                    NVL(:P_DTINICIOVERIFICACAO, TRUNC(SYSDATE, 'iw') - 7) DTINICIOVERIFICACAO, /* Busca o inicio da semana anterior */
                    NVL(:P_DTFINALVERIFICACAO, TRUNC(SYSDATE, 'iw') - 1/86400) DTFINALVERIFICACAO /* Busca o fim da semana anterior */
                FROM
                    GPE_VINCULOM VM
                    JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.NRORG = VM.NRORG
                        AND VH.DTMESCOMPETENC IN (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH VIH
                        WHERE
                            VIH.NRVINCULOM = VH.NRVINCULOM
                            AND VIH.NRORG = VH.NRORG
                            AND TO_CHAR(VIH.DTMESCOMPETENC, 'DD/MM/YYYY') <= :P_DTMESCOMPETENC
                    )
                    LEFT JOIN GPE_PESSOA P ON P.NRORG = VM.NRORG
                      AND P.NRPESSOA = VM.NRPESSOA
                    LEFT JOIN OPERADOR OP ON OP.NRORG = VM.NRORG
                      AND OP.NRPARCNEGOCIO = P.NRPARCNEGOCIO
                    LEFT JOIN COMUNICAPARC CP ON CP.NRORG = VM.NRORG
                      AND CP.NRPARCNEGOCIO = P.NRPARCNEGOCIO
                      AND CP.CDFORMACOMU = '05'
                WHERE
                    VM.NRORG = :P_NRORG
                    AND ( VM.DTRESCISAOVINC IS NULL
                          OR VM.DTRESCISAOVINC > LAST_DAY(SYSDATE) )
                    AND ( INSTR(:P_NRVINCULOM_LIST, ','
                                                  || VM.NRVINCULOM
                                                  || ',') > 0
                          OR :P_NRVINCULOM_LIST IS NULL )
                    AND ( INSTR(:P_NRESTRUTLEGAL_LIST, ','
                                                     || VH.NRESTRUTLEGAL
                                                     || ',') > 0
                          OR :P_NRESTRUTLEGAL_LIST IS NULL )
                    AND ( INSTR(:P_NRESTRUTGEREN_LIST, ','
                                                     || VH.NRESTRUTGEREN
                                                     || ',') > 0
                          OR :P_NRESTRUTGEREN_LIST IS NULL )
                    AND ( INSTR(:P_NRTIPOVINCULOM_LIST, ','
                                                     || VM.NRTIPOVINCULOM
                                                     || ',') > 0
                          OR :P_NRTIPOVINCULOM_LIST IS NULL )
            ),
            
            DIAS_OCORRENCIA AS (
                SELECT
                    VINCULOS.NRVINCULOM,
                    PO.NRPROGOCORRENCIA,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMOTIVOOCORFREQ,
                    MO.DSMOTIVOOCORFREQ,
                    DT.DTDATA,
                    PO.DTINICIOPROGOCOR,
                    PO.DTFIMPROGOCOR,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRHORDIAM,
                    PO.DSOBSERVACAO,
                    PO.NRMARCACAOPONTO
                FROM VINCULOS
                JOIN DATA DT ON DT.DTDATA BETWEEN VINCULOS.DTINICIOVERIFICACAO AND VINCULOS.DTFINALVERIFICACAO
                JOIN PTO_PROGOCORRENCIA PO ON PO.NRORG = VINCULOS.NRORG
                    AND PO.NRVINCULOM = VINCULOS.NRVINCULOM
                    AND DT.DTDATA BETWEEN PO.DTINICIOPROGOCOR AND NVL(PO.DTFIMPROGOCOR, DT.DTDATA)
                    AND PO.IDATIVO = 'S'
                    AND PO.IDPROGOCORCONFIRMADA = 'S'
                    AND SUBSTR(PO.IDDIASSEMANA, TO_CHAR(DT.DTDATA, 'D'), 1) = '1'
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = VINCULOS.NRORG
                    AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
            ),
            
            BATIDAS_DIA AS (
                SELECT
                    VINCULOS.*,
                    MARC.DTHRAJUSTADA,
                    MARC.IDDIAOCORRENCIA,
                    MARC.NRMARCACAOPONTO,
                    CASE
                        WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                            TRUNC(DTHRAJUSTADA) - 1
                        WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                            TRUNC(DTHRAJUSTADA) + 1
                        ELSE
                            TRUNC(DTHRAJUSTADA)
                    END DATA_APURACAO
                FROM
                    VINCULOS
                    JOIN GPE_MARCACAOPONTO MARC ON MARC.NRORG = VINCULOS.NRORG
                        AND MARC.NRVINCULOM = VINCULOS.NRVINCULOM
                        AND (
                            ( MARC.NRCOLETOR NOT IN (
                                SELECT NRCOLETOR
                                FROM GPE_PARAMORG
                                UNPIVOT (NRCOLETOR FOR NRCOLETOR_TIPO IN (NRCOLETORMARCGERSIST, NRCOLETORMARCVIRADIA, NRCOLETORMARCPROJETADA ))
                                WHERE NRORG = VINCULOS.NRORG
                            ) AND :P_NRCOLETOR_LIST IS NULL ) OR
                            INSTR(:P_NRCOLETOR_LIST, ',' || MARC.NRCOLETOR || ',') > 0
                        ) AND (
                            CASE
                                WHEN MARC.IDDIAOCORRENCIA IN ('FA', 'DA') THEN
                                    TRUNC(MARC.DTHRAJUSTADA) - 1
                                WHEN MARC.IDDIAOCORRENCIA IN ('FS', 'DS') THEN
                                    TRUNC(MARC.DTHRAJUSTADA) + 1
                                ELSE
                                    TRUNC(MARC.DTHRAJUSTADA)
                            END
                        ) BETWEEN VINCULOS.DTINICIOVERIFICACAO AND VINCULOS.DTFINALVERIFICACAO
                ORDER BY
                    VINCULOS.NMVINCULOM,
                    MARC.DTHRAJUSTADA
            ),
            
            TOTALIZADORES AS (
                SELECT
                    VINCULOS.*,
                    DT.DTDATA,
                    CASE
                        WHEN OC_COMP_ABONO.NRVINCULOM IS NULL AND NVL(QTD_MARCACOES.VALOR, 0) < NVL(QTD_MARCACOES_HORARIO.VALOR, 0) THEN
                            'Quantidade de marcações em ' || TO_CHAR(DT.DTDATA, 'DD/MM/YYYY') || ' (' || NVL(QTD_MARCACOES.VALOR, 0) || ') é inferior ao previsto'
                        ELSE
                            NULL
                    END QTD_MARCACOES_MENOR
                FROM
                    VINCULOS
                    JOIN DATA DT ON DT.DTDATA BETWEEN VINCULOS.DTINICIOVERIFICACAO AND VINCULOS.DTFINALVERIFICACAO
                    
                    LEFT JOIN (
                        SELECT
                            NRVINCULOM,
                            DTDATA
                        FROM DIAS_OCORRENCIA
                        WHERE NRTIPOPROGOCORRENCIA IN (2, 27, 4, 16, 17, 23, 5, 15)
                        GROUP BY NRVINCULOM, DTDATA
                    ) OC_COMP_ABONO ON OC_COMP_ABONO.NRVINCULOM = VINCULOS.NRVINCULOM
                        AND OC_COMP_ABONO.DTDATA = DT.DTDATA
                        
                    LEFT JOIN (
                        SELECT
                            NRVINCULOM,
                            DTDATA,
                            NRHORDIAM
                        FROM DIAS_OCORRENCIA
                        WHERE NRTIPOPROGOCORRENCIA IN (29)
                        GROUP BY NRVINCULOM, DTDATA, NRHORDIAM
                    ) OC_TROCA_HORARIO ON OC_TROCA_HORARIO.NRVINCULOM = VINCULOS.NRVINCULOM
                        AND OC_TROCA_HORARIO.DTDATA = DT.DTDATA
                                                                     
                    LEFT JOIN GPE_ALTEESCALA AE ON AE.NRORG = VINCULOS.NRORG
                        AND AE.NRVINCULOM = VINCULOS.NRVINCULOM
                        AND AE.NRTURNO IS NOT NULL
                        AND AE.DTINIESCALA = (
                        SELECT
                            MAX(DTINIESCALA)
                        FROM
                            GPE_ALTEESCALA
                        WHERE
                            NRORG = AE.NRORG
                            AND NRVINCULOM = AE.NRVINCULOM
                            AND DT.DTDATA BETWEEN DTINIESCALA AND NVL(DTFIMESCALA, DT.DTDATA)
                    )
                    LEFT JOIN GPE_TURNO T ON T.NRORG = VINCULOS.NRORG
                        AND T.NRTURNO = AE.NRTURNO
                                             
                    LEFT JOIN GPE_HORARIOTURNO HT ON HT.NRORG = VINCULOS.NRORG
                        AND HT.NRTURNO = AE.NRTURNO
                        AND NRSEQHORARIO = (
                        SELECT
                            MOD((DT.DTDATA - TRUNC(T.DTDATABASE)), COUNT(*)) + 1
                        FROM
                            GPE_HORARIOTURNO
                        WHERE
                            NRORG = HT.NRORG
                            AND NRTURNO = HT.NRTURNO
                    )
                    
                    LEFT JOIN (
                        SELECT
                            NRORG,
                            NRHORDIAM,
                            COUNT(*) AS VALOR
                        FROM GPE_INTERVALOSHORARIO
                        UNPIVOT (MARCACAO FOR ENTR_SAIDA IN (HRENTRADA, HRSAIDA))
                        WHERE NRORG = :P_NRORG
                        GROUP BY
                            NRORG,
                            NRHORDIAM
                    ) QTD_MARCACOES_HORARIO ON QTD_MARCACOES_HORARIO.NRHORDIAM = NVL(OC_TROCA_HORARIO.NRHORDIAM, HT.NRHORDIAM)
                    
                    LEFT JOIN GPE_HORDIAH HH ON HH.NRORG = VINCULOS.NRORG
                        AND HH.NRHORDIAM = NVL(OC_TROCA_HORARIO.NRHORDIAM, HT.NRHORDIAM)
                        AND HH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_HORDIAH
                        WHERE
                            NRORG = HH.NRORG
                            AND NRHORDIAM = HH.NRHORDIAM
                            AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                    
                    LEFT JOIN (
                        SELECT
                            NRORG,
                            NRVINCULOM,
                            DATA_APURACAO,
                            COUNT(*) AS VALOR
                        FROM BATIDAS_DIA
                        GROUP BY
                            NRORG,
                            NRVINCULOM,
                            DATA_APURACAO
                    ) QTD_MARCACOES ON QTD_MARCACOES.NRVINCULOM = VINCULOS.NRVINCULOM
                                           AND QTD_MARCACOES.DATA_APURACAO = DT.DTDATA
            )
            
            SELECT
                OCORRENCIAS.NRORG,
                OCORRENCIAS.NRVINCULOM,
                OCORRENCIAS.NMVINCULOM,
                OCORRENCIAS.NRTIPOVINCULOM,
                OCORRENCIAS.NRESTRUTLEGAL,
                OCORRENCIAS.NRESTRUTGEREN,
                OCORRENCIAS.EMAIL,
                OCORRENCIAS.TIPO_OCORRENCIA,
                OCORRENCIAS.DATAS
            FROM
                (
                    SELECT
                        NRORG,
                        NRVINCULOM,
                        NMVINCULOM,
                        NRTIPOVINCULOM,
                        NRESTRUTLEGAL,
                        NRESTRUTGEREN,
                        EMAIL,
                        TIPO_OCORRENCIA  AS TIPO_OCORRENCIA,
                        LISTAGG(TO_CHAR(DTDATA, 'DD/MM/YYYY'), ',') WITHIN GROUP (ORDER BY DTDATA) DATAS
                    FROM
                        TOTALIZADORES
                    UNPIVOT (OBSERVACAO FOR TIPO_OCORRENCIA IN ( QTD_MARCACOES_MENOR ))
                    GROUP BY NRORG, NRVINCULOM, NMVINCULOM, NRTIPOVINCULOM, NRESTRUTLEGAL, NRESTRUTGEREN, EMAIL, TIPO_OCORRENCIA
                ) OCORRENCIAS
            ORDER BY
                NMVINCULOM
            NULLS LAST
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute([
            'P_NRORG' => Utils::getParamters()->getNrorg(),
            'P_DTMESCOMPETENC' => Utils::getParamters()->getDtmescomptencDiaCorrente(),
            'P_NRVINCULOM_LIST' => Utils::arrayToStringForQueryParam($nrVinculomList),
            'P_NRESTRUTLEGAL_LIST' => Utils::arrayToStringForQueryParam($nrEstruturamLegal),
            'P_NRESTRUTGEREN_LIST' => Utils::arrayToStringForQueryParam($nrEstruturamGeren),
            'P_NRTIPOVINCULOM_LIST' => Utils::arrayToStringForQueryParam($nrTipoVinculomList),
            'P_NRCOLETOR_LIST' => Utils::arrayToStringForQueryParam($nrColetorList),
            'P_DTINICIOVERIFICACAO' => $dtInicioVerificacao,
            'P_DTFINALVERIFICACAO' => $dtFinalVerificacao,
        ]);

        return $std->fetchAll();
    }
}
