<?php

namespace Ems\Services;

use Ems\Repository\{
    ReconhecimentoFacialRepository
};
use Ems\Util\{
    Base64Util,
    ExpectedException,
    RequestUtils,
    Utils
};
use GuzzleHttp\Client;
use Util\{
    Environment,
    NovoCodigo,
    DateUtil
};
use Ems\Services\{
    ParceiroNegocioService,
};
use EMS\Util\Utilities;


class ReconhecimentoFacialService {
    /** @var Environment */
    protected $environment;

    /** @var Client */
    protected $http;

    protected $accountInfo = null;

    /** @var string */
    protected $reconhecimentoFacialUrl = null;

    /** @var ReconhecimentoFacialRepository */
    protected $reconhecimentoFacialRepository;

    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;


    /** @var Utilities */
    protected $utilities;

    public function __construct(
        Environment                    $environment,
        string                         $reconhecimentoFacialUrl,
        ReconhecimentoFacialRepository $reconhecimentoFacialRepository,
        ParceiroNegocioService         $parceiroNegocioService
    ) {
        $this->environment                    = $environment;
        $this->reconhecimentoFacialUrl        = $reconhecimentoFacialUrl;
        $this->reconhecimentoFacialRepository = $reconhecimentoFacialRepository;
        $this->parceiroNegocioService         = $parceiroNegocioService;

        $nrOrg = $this->environment->getNrOrg();

        if (!$nrOrg) {
            $this->setNrOrgFromHeader();
        }

        $this->utilities = Utils::getInstance('Utilities');
    }


    public function getUri(
        string $route
    ): string {
        $uri = $this->reconhecimentoFacialUrl . $route;
        return $uri;
    }

    public function helloWorld(): array {

        $uri = $this->getUri('helloWorld');

        $headers = [
            'Content-Type' => 'application/json'
        ];

        $body = [];

        $responseBody = RequestUtils::request($uri, $body, 'GET', $headers);
        

        if (!$responseBody) {
            throw new ExpectedException('Não foi possível fazer o upload do arquivo no Certisign!');
        }

        return $responseBody;
    }

    public function encodaImagem(string $base64Image1) {

        $uri = $this->getUri('encoda-imagem/');

        // Decodificar a string Base64 para obter os dados binários da imagem
        $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64Image1));

        // Criar um cliente GuzzleHttp
        $client = new Client();

        // Enviar a requisição usando o cliente GuzzleHttp
        $response = $client->request('POST', $uri, [
            'multipart' => [
                [
                    'name' => 'arquivo',
                    'contents' => $imageData,
                    'filename' => 'image.jpg' // Nome do arquivo, você pode ajustar conforme necessário
                ]
            ]
        ]);

        $content = $response->getBody()->getContents();
        

        if (!$content) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        $response = json_decode($content);

        if(!$response->resultado || $response->resultado == false) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        return $response->resultado;
    }

    public function verificaRostoEncodaECompara(string $base64Image1, string $base64Image2) {

        $uri = $this->getUri('verifica-rosto-encoda-compara/');

        // Decodificar a string Base64 para obter os dados binários da imagem
        $imageData1 = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64Image1));
        $imageData2 = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64Image2));


        // Criar um cliente GuzzleHttp
        $client = new Client();

        // Enviar a requisição usando o cliente GuzzleHttp
        $response = $client->request('POST', $uri, [
            'multipart' => [
                [
                    'name' => 'arquivo1',
                    'contents' => $imageData1,
                    'filename' => 'image.jpg' // Nome do arquivo, você pode ajustar conforme necessário
                ],
                [
                    'name' => 'arquivo2',
                    'contents' => $imageData2,
                    'filename' => 'image2.jpg' // Nome do arquivo, você pode ajustar conforme necessário
                ]
            ]
        ]);


        $content = $response->getBody()->getContents();
    

        if (!$content) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        $response = json_decode($content);

        if(!$response->resultado) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        return $response;
    }

    public function saveHashFacial(string $nrparcnegocio, string $base64Image) {
        $nrOrg = $this->environment->getNrOrg();

        /*
        $teste = $this->helloWorld();
        var_dump($teste);die;*/

        $hashContent = $this->encodaImagem($base64Image);

        $hash = implode(',', $hashContent);

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrparcnegocio,
            'P_HASHFACIAL' => $hash
        ];

        $this->reconhecimentoFacialRepository->saveHashFacial($params);

        return true;
    }

    public function reconhecimentoFacial(string $base64Image, string $hashConhecidos) {
        $uri = $this->getUri('verifica-rosto-compara-com-hashs-conhecidos/');

        // Decodificar a string Base64 para obter os dados binários da imagem
        $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64Image));

        // Criar um cliente GuzzleHttp
        $client = new Client();

        // Enviar a requisição usando o cliente GuzzleHttp
        $response = $client->request('POST', $uri, [
            'multipart' => [
                [
                    'name' => 'hashsConhecidos',
                    'contents' => $hashConhecidos
                ],
                [
                    'name' => 'arquivo',
                    'contents' => $imageData,
                    'filename' => 'image.jpg'
                ]
            ]
        ]);

        $content = $response->getBody()->getContents();


        if (!$content) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        $response = json_decode($content);

        if(!$response->resultado) {
            throw new ExpectedException('Não foi encontrado resultado!');
        }

        return $response;
    }

}