<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\FPA\Services\CalculoFolhaPercService;

class CalculoFolhaPercController extends Crud {

    protected $dataSourceNameGet = 'getCalcPerc';
    protected $CalculoFolhaPercService;
    protected $parameterBagColumns = ['P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC' ,'P_CDOPERADOR'];

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, CalculoFolhaPercService $CalculoFolhaPercService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->calculofolhapercservice = $CalculoFolhaPercService;
        parent::__construct($dataSourceManager);
    }

    public function getCalcPerc (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }
            
            if($row['DSORDEM'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['NOMEFINAL']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['NOMEINICIAL']);
                
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['NOMEFINAL']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }

            if($row['DSORDEM'] === 'LISTA_TOMADOR'){
                $row['LIST_NRESTRUTURAMSUP'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['LIST_NRESTRUTURAMSUP'] as $chave => $value)
                    $row['LIST_NRESTRUTURAMSUP'][$chave] = intval($value);
                
                $row['NMESTRUTURAM'] = $row['NMESTRUTLEGAL'];
                $row['NMESTRUTURAMSUP'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['NOMEFINAL']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['NOMEFINAL']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }

            if($row['DSORDEM'] == 'LISTA_LOTE'){
                $row['NRLOTEVINC_LIST'] = explode(",", $row['NOMEINICIAL']);
                foreach($row['NRLOTEVINC_LIST'] as $chave => $value)
                    $row['NRLOTEVINC_LIST'][$chave] = intval($value);
                
                $row['LIST_NMLOTEVINC'] = explode(",", $row['LIST_NMLOTEVINC']);
            }
            
            if(isset($row['NREVENTOM'])){
                $row['NREVENTOM'] = explode(",", $row['NREVENTOM']);
                
                foreach($row['NREVENTOM'] as $chave => $value)
                    $row['NREVENTOM'][$chave] = intval($value);
                    
                $row['NMEVENTOH'] = explode(",", $row['NMEVENTOH']);
            }
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
    public function setCalcPerc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_CALCPERC";

            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMCALCPERC', $row)) {
                    if ($row['NRFORMCALCPERC'] === '') {
                        $row['NRFORMCALCPERC'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMCALCPERC', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMCALCPERC'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMCALCPERC', 1, 12, true); 
                }
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                if($row['NREVENTOM'] != ""){
                    $row['NREVENTOM'] = implode(',',$row['NREVENTOM']);
                }
                
                // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
    
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if((is_array($row['DSFIM'])) && (count($row['DSFIM']) > 0)) {
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
    
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if((is_array($row['DSFIM'])) && (count($row['DSFIM']) > 0)) {
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }

                if($row['DSORDEM'] === "LISTA_TOMADOR"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(isset($row['DSFIM']) && is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }

                if($row['DSORDEM'] === "LISTA_LOTE"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }

                $row['NOMEINICIAL'] = $row['DSINICIO'];
                $row['NOMEFINAL']   = isset($row['DSFIM']) ? $row['DSFIM'] : null;
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function calculoPerc(DTO\Request\DataSet $request, DTO\Response $response){
        set_time_limit(0);
        
        error_reporting(E_ERROR | E_PARSE);
        
        $dataSet = $request->getDataSet()->getRows()[0];

        try{
            $resultado = $this->calculofolhapercservice->calculoPerc($dataSet);

            if($resultado){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($resultado, DTO\Response\Message::TYPE_MESSAGE));
            }
            else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Nenhum vinculo encontrado.', DTO\Response\Message::TYPE_MESSAGE));
            }
        } catch(\Exception $e){
            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage()));
        }
    }

    public function deleteCalcPerc(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_CALCPERC";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = [];
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}