<?php

namespace HCM\FPA\Controllers;

use HCM\FPA\Services\VincEventoService;
use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class VincEventoController extends Crud {

    protected $dataSourceName = 'fpa_vincevento';
    protected $dataSourceName_get = 'getVincEvento';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC', 'P_RESCISAO','P_ULTIMODIAMES');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, VincEventoService $vincEventoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->vincEventoService = $vincEventoService;
        parent::__construct($dataSourceManager);
    }

    public function getVincEvento (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $recisaofiltro = false;
        
        foreach ($filterData->getConditions() as $value) {
            if($value['columnName'] == "P_RESCISAO") {
                $recisaofiltro = true;
            }
        }
        if(!$recisaofiltro) {
            $filterData->addCondition('P_RESCISAO','=', 'A');
        }
        $filterData->addCondition('P_ULTIMODIAMES','=',$this->hcmParameters->getUltimoDiaMes());
        
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setVincEvento (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $msg = array();
            $disponibilidade = true;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->vincEventoService->verificaEventoBloqueado($this->hcmParameters->getNrorgTrabalho(), $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                if ($disponibilidade === true) {
                    if (array_key_exists('NRORG', $row)) {
                        if ($row['NRORG'] === '') {
                            $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                        }
                    } else {
                        $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                    }
    
                    if (!array_key_exists('NRVALORREFERENCIAM', $row)) {
                        $row['NRVALORREFERENCIAM'] = '';
                    }
    
                    if (!array_key_exists('DSOBSERVACAO', $row)) {
                        $row['DSOBSERVACAO'] = '';
                    }
    
                    if (!array_key_exists('NRTPAGRUPACALC', $row)) {
                        $row['NRTPAGRUPACALC'] = '';
                    }
    
                    if (!array_key_exists('NRESTRUTURAM', $row)) {
                        $row['NRESTRUTURAM'] = '';
                    }
                    
                    if (!array_key_exists('NRTIPOINCIDE', $row)) {
                        $row['NRTIPOINCIDE'] = '';
                    }
    
                    if (array_key_exists('NRVINCEVENTO', $row)) {
                        if ($row['NRVINCEVENTO'] === '') {
                            $row['NRVINCEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_VINCEVENTO', 1, 12, true);
                        }
                    } else {
                        $row['NRVINCEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_VINCEVENTO', 1, 12, true);
                    }
                    
                    $verificaParcela =  $this->verificaParcela($row);
                    if($verificaParcela == 'E'){
                        $msg[] = array('msg' => "Operação bloqueada. Para inserir um Tipo de Referência cujo valor seja igual a Parcela, favor selecionar uma data de Início e Fim de Vigência.");
                        continue;
                    }
                    
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteVincEvento(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if ($this->vincEventoService->verificaEventoRelacBeneficiario($row['NRORG'], $row['NRVINCEVENTO'])) {
                    $response->addMessage(new DTO\Response\Message(Messages::HCM01000_FPA_VINC_EVENT_BENEF_ERROR), DTO\Response\Message::TYPE_ERROR);
                    return;
                }
                parent::delete($request, $response);
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cadAutoVincEvento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrvinculom_list = $row['NRVINCULOM_LIST'];
                $dtinivigencia = isset($row['DTINIVIGENCIA']) ? $row['DTINIVIGENCIA'] : null;
                $dtfimvigencia = isset($row['DTFIMVIGENCIA']) ? $row['DTFIMVIGENCIA'] : null;
                $idtipomovimento = isset($row['IDTIPOMOVIMENTO']) ? $row['IDTIPOMOVIMENTO'] : null;
                $nreventom = isset($row['NREVENTOM_LIST']) ? $row['NREVENTOM_LIST'] : null;
                $vrpisoevento = isset($row['VRPISOEVENTO']) ? $row['VRPISOEVENTO'] : null;;
                $vrtetoevento = isset($row['VRTETOEVENTO']) ? $row['VRTETOEVENTO'] : null;
                $nrvalreferencia =  isset($row['NRVALORREFERENCIAM']) ? $row['NRVALORREFERENCIAM'] : null;
                $vrpercent = isset($row['VRPERCENT']) ? $row['VRPERCENT'] : null;
                $vrmultiplic = isset($row['VRMULTIPLIC']) ? $row['VRMULTIPLIC'] : null;
                $vrdivide = isset($row['VRDIVIDE']) ? $row['VRDIVIDE'] : null;
                $vreventofixo = isset($row['VREVENTOFIXO']) ? $row['VREVENTOFIXO'] : null;
                $idrefgerar = isset($row['IDREFGERAR']) ? $row['IDREFGERAR'] : null;
                $vrreffixo = isset($row['VRREFFIXO']) ? $row['VRREFFIXO'] : null;
                $idretreferencia = isset($row['IDRETREFERENCIA']) ? $row['IDRETREFERENCIA'] : null;
                $periodicidademes = isset($row['PERIODICIDADEMES']) ? $row['PERIODICIDADEMES'] : null;
                $nrestruturam = isset($row['NRESTRUTURAM']) ? $row['NRESTRUTURAM'] : null;
                $idgeraafastado = isset($row['IDGERAAFASTADO']) ? $row['IDGERAAFASTADO'] : null;

                $this->vincEventoService->cadastroAutomatico($nrorg, $dtinivigencia, $dtfimvigencia, $idtipomovimento, $nreventom,
                    $vrpisoevento, $vrtetoevento, $nrvalreferencia, $vrpercent, $vrmultiplic, $vrdivide, $vreventofixo,
                    $idrefgerar, $vrreffixo, $idretreferencia, $periodicidademes, $nrvinculom_list, $nrestruturam, $idgeraafastado);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function verificaParcela($row){
        if($row['IDREFGERAR'] == "PARCELA"){
            if($row['DTFIMVIGENCIA'] == null || $row['DTFIMVIGENCIA'] == ''){
                return 'E';
            }
        }
    }
}