<?php
 
namespace HCM\Geral\Logic;

use HCM\Geral\Repository\ImportacaoPlanilhaVinculoRepository;

use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\Util;

class ImportacaoPlanilhaVinculoPessoaLogic {

    /** @var \Doctrine\ORM\EntityManager */
    protected $entityManager;
    /** @var \HCM\Util\HCMParameters */
    protected $hcmParameters;
    /** @var \Doctrine\DBAL\Connection */
    protected $connection;
    /** @var \Doctrine\ORM\Configuration */
    protected $config;
    

    private $arrDados;
    private $nrorg;
    private $nrorgpadrao;
    private $parcnegocio;
    private $gpePessoa;
    private $gpePessoah;
    private $enderecoParc;
    private $contCorrParc;
    private $formacaoAcad;
    private $comunicaParc;
    private $operador;
    
    private $importacaoPlanilhaVinculoRepository;

    public function __construct(EntityManager $entityManager, Connection $connection, Configuration $config, HCMParameters $hcmParameters, Array $arrayDados) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->connection    = $connection;
        $this->config        = $config;
        $this->arrDados      = $arrayDados;
        $this->nrorg         = $this->hcmParameters->getNrorgTrabalho();
        $this->nrorgpadrao   = $this->hcmParameters->getNrorgPadrao();
        $this->parcnegocio   = Repositories::PARCNEGOCIO;
        $this->gpePessoa     = Repositories::GPE_PESSOA;
        $this->gpePessoah    = Repositories::GPE_PESSOAH;
        $this->enderecoParc  = Repositories::ENDERECOPARC;
        $this->contCorrParc  = Repositories::CONTCORRPARC;
        $this->formacaoAcad  = Repositories::DES_FORMACAO;
        $this->comunicaParc  = Repositories::COMUNICAPARC;
        $this->operador      = Repositories::OPERADOR;
        
        $this->importacaoPlanilhaVinculoRepository = new ImportacaoPlanilhaVinculoRepository($this->connection, $this->entityManager, $this->hcmParameters);
    }

    public function criaDados(&$arrayMsgError) {
        try{
            list($nmvinculom,       $cdestacivil,      $nrcpfpessoa,      $nrctpspessoa,     $nrseriectpspes,
                 $sgufctpspes,      $dtctpspessoa,     $nrpispaseppes,    $dtinscpispasep,   $nrrgpessoa,
                 $cdexrgpessoa,     $sgufrgpessoa,     $dtexrgpessoa,     $nrtitueleipes,    $nrsecaeleipes,
                 $nrzonaeleipes,    $nrcarthabpes,     $dtvalhabcartpes,  $dscateghabcart,   $nrcertresepes,
                 $nrcatmilitarpe,   $nrracapessoa,     $cdpais,           $sgestado,         $dtnascpessoa,
                 $idsexopessoa,     $cdbanco,          $cdagencia,        $cdcontcorr,       $nrtpcontcorr,
                 $nrnacionalidade,  $cdpais_end,       $sgestado_end,     $cdmunicipio_end,  $cdlogradouro_end,
                 $dsendereco,       $nrimovelendereco, $nmbairroendereco, $dscompleendereco, $nrcependereco,
                 $nrgrauinstr,      $idgrupsangupes,   $idfatorrhpes,     $nrinscrconsreg,   $nrcondfispes,
                 $telprincipal,     $dsemailoper,      $newdtmescompetenc)                                  = $this->setaDados();

            $arrayPessoa = [
                'NMPESSOA'        => strtoupper(Util::removeCaracteresEspeciais($nmvinculom)),
                'NRCPFPESSOA'     => $nrcpfpessoa,
                'DTNASCPESSOA'    => $dtnascpessoa,
                'IDSEXOPESSOA'    => $idsexopessoa,
                'NRORG'           => $this->nrorg,
                'EMAIL'           => $dsemailoper,
                'NRPISPASEPPES'   => $nrpispaseppes,
                'NRCTPSPESSOA'    => $nrctpspessoa,
                'NRSERIECTPSPES'  => $nrseriectpspes,
                'SGUFCTPSPES'     => $sgufctpspes,
                'DTCTPSPESSOA'    => $dtctpspessoa,
                'NRRGPESSOA'      => $nrrgpessoa,
                'DTINSCPISPASEP'  => $dtinscpispasep,
                'NRCONDFISPES'    => $nrcondfispes,
                'CDESTACIVIL'     => $cdestacivil,
                'NRTITUELEIPES'   => $nrtitueleipes,
                'NRSECAELEIPES'   => $nrsecaeleipes,
                'NRZONAELEIPES'   => $nrzonaeleipes,
                'NRCARTHABPES'    => $nrcarthabpes,
                'DTVALHABCARTPES' => $dtvalhabcartpes,
                'DSCATEGHABCART'  => $dscateghabcart,
                'NRCATMILITARPE'  => $nrcatmilitarpe,
                'NRCERTRESEPES'   => $nrcertresepes,
                'NRRACAPESSOA'    => $nrracapessoa,
                'IDGRUPSANGUPES'  => $idgrupsangupes,
                'IDFATORRHPES'    => $idfatorrhpes,
                'CDPAIS'          => $cdpais   ?? $cdpais_end,
                'SGESTADO'        => $sgestado ?? $sgestado_end,
                'CDMUNICIPIO'     => $cdmunicipio_end,
                'CDEXRGPESSOA'    => $cdexrgpessoa,
                'SGUFRGPESSOA'    => $sgufrgpessoa,
                'DTEXRGPESSOA'    => $dtexrgpessoa,
                'NRNACIONALID'    => $nrnacionalidade,
                'NRGRAUINSTR'     => $nrgrauinstr,
                'NRINSCRCONSREG'  => $nrinscrconsreg,
                'IDCOTAPCD'       => 'N'
            ];

            $arrayEndereco = [
                'CDLOGRADOURO'     => $cdlogradouro_end,
                'DSLOGRADOURO'     => $cdlogradouro_end,
                'CDPAIS'           => $cdpais_end,
                'SGESTADO'         => $sgestado_end,
                'CDMUNICIPIO'      => $cdmunicipio_end,
                'NMBAIRROENDERECO' => $nmbairroendereco,
                'CDTIPOENDERECO'   => 'PRINCIPAL',
                'NRCEPENDERECO'    => $nrcependereco,
                'DSCOMPLEENDERECO' => $dscompleendereco,
                'NRIMOVELENDERECO' => $nrimovelendereco,
                'DSENDERECO'       => $dsendereco
            ];
            
            $pessoaMesmoCPF = $this->validaCpfExistePessoa($arrayPessoa, false);

            if($pessoaMesmoCPF) {
                $arrayMsgError[] = $pessoaMesmoCPF;
                return;
            }

            $pessoa = $this->verificaIndex($arrayPessoa);

            //INSERE PARCEIRO NEGOCIO
            $parcnegocio = $this->parcNegocioNovo($this->nrorg, $pessoa['NMPESSOA'], $pessoa['NMPESSOA'],'PESSOA','S','N','CPF', $pessoa['NRCPFPESSOA'],Util::datadeString($pessoa['DTNASCPESSOA'],true));
            
            //DEFINE NRPARCNEGOCIO
            if($parcnegocio)
                $nrparcnegocio = $parcnegocio->getNrparcnegocio();
            else
                throw new \Exception("Falha na criação do parceiro negocio.");

            //INSERE PESSOA
            list($pessoam, $pessoah) = $this->pessoaNova($this->nrorg, $nrparcnegocio, $newdtmescompetenc, $pessoa['NMPESSOA'], $pessoa['DSAPELIDOPESS'], 
                                                            $pessoa['NRCERTNASCPES'], Util::datadeString($pessoa['DTNASCPESSOA'],true),$pessoa['IDSEXOPESSOA'], $pessoa['NRCONDFISPES'], $pessoa['NRCPFPESSOA'], 
                                                            $pessoa['NRCTPSPESSOA'], $pessoa['NRSERIECTPSPES'], $pessoa['SGUFCTPSPES'], Util::datadeString($pessoa['DTCTPSPESSOA'],true), $pessoa['NRPISPASEPPES'], 
                                                            Util::datadeString($pessoa['DTINSCPISPASEP'],true), $pessoa['NRINSCINSSPES'], $pessoa['NRINSCISSPES'], $pessoa['CDESTACIVIL'], $pessoa['NRCERTCASAPES'], 
                                                            Util::datadeString($pessoa['DTCASAPESSOA'],true), $pessoa['IDISENTITUELEI'], $pessoa['NRTITUELEIPES'], $pessoa['NRSECAELEIPES'], $pessoa['NRZONAELEIPES'],
                                                            $pessoa['NRCARTHABPES'], Util::datadeString($pessoa['DTHABCARTPES'],true), Util::datadeString($pessoa['DTVALHABCARTPES'],true), $pessoa['DSCATEGHABCART'],
                                                            $pessoa['NRCATMILITARPE'], $pessoa['NRCERTRESEPES'], $pessoa['DSCATEGCERTRESE'], Util::datadeString($pessoa['DTCERTRESEPES'],true), $pessoa['CDEXCERTRESEPES'],
                                                            $pessoa['NRRACAPESSOA'], $pessoa['IDGRUPSANGUPES'], $pessoa['IDFATORRHPES'], $pessoa['CDPAIS'], $pessoa['SGESTADO'], $pessoa['CDMUNICIPIO'], $pessoa['NRRGPESSOA'],
                                                            $pessoa['CDEXRGPESSOA'], $pessoa['SGUFRGPESSOA'], $pessoa['DSLOCALEXRGPES'], Util::datadeString($pessoa['DTEXRGPESSOA'],true), $pessoa['NRNACIONALID'], 
                                                            $pessoa['NRGRAUINSTR'], $pessoa['DSPRONOMETRAT'], $pessoa['NRANOCHEGAPAIS'], $pessoa['NRDOCUESTRANG'], $pessoa['IDPESNATURALIZA'], 
                                                            Util::datadeString($pessoa['DTNATURALIZAPES'],true), $pessoa['IDPOSVISTOPERM'],  Util::datadeString($pessoa['DTPERMAVISTO'],true), $pessoa['NRCONSELHOREG'], 
                                                            $pessoa['NRINSCRCONSREG'], Util::datadeString($pessoa['DTEXCONSREG'],true), $pessoa['SGUFCONSREG'], null, $pessoa['NRLIVROCERTNASC'], $pessoa['NRFOLHACERTNASC'], 
                                                            $pessoa['DSCOMPLCERTNASC'], $pessoa['NRREGCTPSPES'], $pessoa['NRCLASSESTRANG'], $pessoa['DSEXPDOCESTR'],  $pessoa['DTCHEGADAPAIS'], $pessoa['IDCOTAPCD']);
            //DEFINE NRPESSOA
            if($pessoam)
                $nrpessoa = $pessoam->getNrpessoa();
            else if(!$pessoam || !$pessoah)
                throw new \Exception("Falha na criação da pessoa(criaDados).");

            //INSERE FORMAÇÃO ACADEMICA DO PARCEIRO
            $this->novoFormacaoAcademica($nrparcnegocio, $pessoa);

            //INSERE ENDERECO DO PARCEIRO
            $endereco = $this->verificaIndex($arrayEndereco);
            $endereco['CDLOGRADOURO'] = $this->convertDsLogrToCdlogr($endereco['DSLOGRADOURO']);

            if(!$endereco['CDLOGRADOURO']){
                $endereco['DSLOGRADOURO'] = "Rua";
                $endereco['CDLOGRADOURO'] = "RUA";
            }

            $this->novoEndereco($nrparcnegocio, $endereco['CDPAIS'], $endereco['SGESTADO'], $endereco['CDMUNICIPIO'], $endereco['CDLOGRADOURO'], $endereco['NMBAIRROENDERECO'],
                                $endereco['CDTIPOENDERECO'], $endereco['NRCEPENDERECO'], $endereco['DSCOMPLEENDERECO'], $endereco['NRIMOVELENDERECO'], $endereco['DSENDERECO'],$endereco['DSREFERENCIAENDE']);

            //INSERE CONTA CORRENTE DO PARCEIRO
            if($cdbanco) {
                $arrayContaCorrente = [
                    'CDBANCO'      => $cdbanco,
                    'CDAGENCIA'    => $cdagencia,
                    'CDCONTCORR'   => $cdcontcorr,
                    'NRTPCONTCORR' => $nrtpcontcorr
                ];

                $contaCorrente = $this->verificaIndex($arrayContaCorrente);
                $this->novoContaCorrente($nrparcnegocio, null, $contaCorrente['CDBANCO'], $contaCorrente['CDAGENCIA'], $contaCorrente['CDCONTCORR'], $contaCorrente['NRTPCONTCORR']);
            }

            //INSERE FORMA COMUNICACAO DO PARCEIRO
            if($telprincipal){
                $arrayFormaComunicacao = [
                    'CDCOMUNICAPARC'   => $telprincipal,
                    'CDPREFIXCOMUPARC' => null,
                    'CDFORMACOMU'      => '01',
                    'NRPARCNEGOCIO'    => $nrparcnegocio
                ];

                $this->novaFormacomu($arrayFormaComunicacao['NRPARCNEGOCIO'], $arrayFormaComunicacao['CDFORMACOMU'], $arrayFormaComunicacao['CDPREFIXCOMUPARC'], $arrayFormaComunicacao['CDCOMUNICAPARC'], null, null);
            }
            
            //CONSIDERA APENAS A EXISTÊNCIA DO E-MAIL PARA CRIAR O OPERADOR.
            if($dsemailoper){
                $senha = md5(str_repeat(strtolower(substr($pessoa['NMPESSOA'],0,1)),5));
                $operador = $this->operadorNovo(
                                $this->nrorg,
                                $pessoa['NMPESSOA'],
                                Util::datadeString($pessoa['DTNASCPESSOA'],true),
                                $pessoa['EMAIL'],
                                $nrparcnegocio,
                                $senha,
                                'A','A','S','N','003','N','N','N','N','000000000000000000000000000000',
                                !empty($pessoa['NMACESSO']) ? $pessoa['NMACESSO'] : $pessoa['NRCPFPESSOA']
                            );
            }

            $operadorFoiCriado = !empty($operador) ? true : false;

            return [$pessoam, $pessoah, $operadorFoiCriado];

        } catch (\Exception $e) {
            $msgError = $e->getMessage();

            $arrayMsgError[] = "Ocorreu um erro ao criar dados da pessoa. Error: $msgError";

            if($this->connection->isTransactionActive()) {
                $this->entityManager->rollback();
            }

            if($pessoam || $pessoah) {
                if (!$this->entityManager->isOpen())
                    $this->entityManager = $this->entityManager->create($this->connection, $this->config);
                
                $this->importacaoPlanilhaVinculoRepository->deleteImpPlanilhaVinculoPessoa($nrpessoa, $this->nrorg, $this->entityManager);
                $this->entityManager->flush();
            }
        }
    }
    
    private function setaDados() {
        return [$this->arrDados['NMVINCULOM'],      $this->arrDados['CDESTACIVIL'],      $this->arrDados['NRCPFPESSOA'],      $this->arrDados['NRCTPSPESSOA'],     $this->arrDados['NRSERIECTPSPES'],
                $this->arrDados['SGUFCTPSPES'],     $this->arrDados['DTCTPSPESSOA'],     $this->arrDados['NRPISPASEPPES'],    $this->arrDados['DTINSCPISPASEP'],   $this->arrDados['NRRGPESSOA'],
                $this->arrDados['CDEXRGPESSOA'],    $this->arrDados['SGUFRGPESSOA'],     $this->arrDados['DTEXRGPESSOA'],     $this->arrDados['NRTITUELEIPES'],    $this->arrDados['NRSECAELEIPES'],
                $this->arrDados['NRZONAELEIPES'],   $this->arrDados['NRCARTHABPES'],     $this->arrDados['DTVALHABCARTPES'],  $this->arrDados['DSCATEGHABCART'],   $this->arrDados['NRCERTRESEPES'],
                $this->arrDados['NRCATMILITARPE'],  $this->arrDados['NRRACAPESSOA'],     $this->arrDados['CDPAIS'],           $this->arrDados['SGESTADO'],         $this->arrDados['DTNASCPESSOA'],
                $this->arrDados['IDSEXOPESSOA'],    $this->arrDados['CDBANCO'],          $this->arrDados['CDAGENCIA'],        $this->arrDados['CDCONTCORR'],       $this->arrDados['NRTPCONTCORR'],
                $this->arrDados['NRNACIONALIDADE'], $this->arrDados['CDPAIS_END'],       $this->arrDados['SGESTADO_END'],     $this->arrDados['CDMUNICIPIO_END'],  $this->arrDados['CDLOGRADOURO_END'],
                $this->arrDados['DSENDERECO'],      $this->arrDados['NRIMOVELENDERECO'], $this->arrDados['NMBAIRROENDERECO'], $this->arrDados['DSCOMPLEENDERECO'], $this->arrDados['NRCEPENDERECO'],
                $this->arrDados['NRGRAUINSTR'],     $this->arrDados['IDGRUPSANGUPES'],   $this->arrDados['IDFATORRHPES'],     $this->arrDados['NRINSCRCONSREG'],   $this->arrDados['NRCONDFISPES'],
                $this->arrDados['TELPRINCIPAL'],    $this->arrDados['DSEMAILOPER'],      $this->arrDados['NEWDTMESCOMPETENC']];
    }

    private function verificaIndex($indexs){
        $arrayindex = ['NMPESSOA','DSAPELIDOPESS','NRCERTNASCPES','DTNASCPESSOA','IDSEXOPESSOA','NRCONDFISPES','NRCPFPESSOA','NRCTPSPESSOA',
                       'NRSERIECTPSPES','SGUFCTPSPES','DTCTPSPESSOA','NRPISPASEPPES','DTINSCPISPASEP','NRINSCINSSPES','NRINSCISSPES','CDESTACIVIL','NRCERTCASAPES',
                       'DTCASAPESSOA','IDISENTITUELEI','NRTITUELEIPES','NRSECAELEIPES','NRZONAELEIPES','NRCARTHABPES','DTHABCARTPES','DTVALHABCARTPES','DSCATEGHABCART',
                       'NRCATMILITARPE','NRCERTRESEPES','DSCATEGCERTRESE','DTCERTRESEPES','CDEXCERTRESEPES','NRRACAPESSOA','IDGRUPSANGUPES','IDFATORRHPES','CDPAIS',
                       'SGESTADO','CDMUNICIPIO','NRRGPESSOA','CDEXRGPESSOA','SGUFRGPESSOA','DSLOCALEXRGPES','DTEXRGPESSOA','NRNACIONALID','NRGRAUINSTR','DSPRONOMETRAT',
                       'NRANOCHEGAPAIS','NRDOCUESTRANG','IDPESNATURALIZA','DTNATURALIZAPES','IDPOSVISTOPERM','DTPERMAVISTO','NRCONSELHOREG','NRINSCRCONSREG',
                       'DTEXCONSREG','SGUFCONSREG','NRLIVROCERTNASC','NRFOLHACERTNASC','DSCOMPLCERTNASC','NRREGCTPSPES','CDLOGRADOURO','NMBAIRROENDERECO',
                       'CDTIPOENDERECO','NRCEPENDERECO','DSCOMPLEENDERECO','NRIMOVELENDERECO','DSENDERECO','DSREFERENCIAENDE','CDBANCO',
                       'CDBANCO','CDAGENCIA','CDCONTCORR','NRTPCONTCORR','EMAIL','NRPESSOA','DTADMISSAOVINC','CDMATRICULA','NRTIPOVINCULOM',
                       'NRVINCULOEMPREG','DTOPCAOFGTS','DTAPOSENTAFGTS','DTPRIMADMISSAO','NMVINCULOM','NRDIASCONTRATOEXP','NRDIASCONTRATOEXPPRO','DTEXAMEMEDICO',
                       'IDCONTRIBUISIND','DTBASEFERIAS','IDREMUNERACAO','IDTPPAGAMENTO','IDMULTVINC','CDCONTRIBINDIVIDUAL','NRTPMOVTRANSFM','DTFIMCONTRDETERMIN',
                       'NRTIPOTRANSFER_SINDICAL','NRESTRUTSIND','DTINIMOVIMENT','NRTIPOTRANSFER_LEGAL','NRESTRUTLEGAL','NRTIPOTRANSFER_GEREN','NRESTRUTGEREN', 'NRTIPOTRANSFER_ADICIONAL', 'NRESTRUTADICIONAL',
                       'NRTPMOVTRANSF','NRSITUFUNCM','DTINISITUFUNC','QTHRLIMITESEPHEXTRA','QTHRLIMITESEPHEXTRA','DTALTESALARIO','NROCORRENCIA','NRTIPOALTERA','VRSALARIO',
                       'CDTIPOSALARIO','NRTPMODALIDSAL','VRDIFERENCAREAL','VRPERCREAL','VRPERCALTERA','NRESCALATRABM','DTINIESCALA','DTFIMESCALA','NRTURNO','QTHRLIMITESEPHEXTRA',
                       'NROCUPACAOM','DTINIOCUPACAO','NRTIPOOCUPACAO','NRTPMODALIDSAL','DTFIMOCUPACAO','INSERE_OPERADOR','NRTIPORELACIONA','DTFIMRELACIONAMENTO','NRTIPODEPENDE','NRNACIONALIDADE',
                       'NRCLASSESTRANG', 'DSEXPDOCESTR', 'DTCHEGADAPAIS', 'NRMOTIVOADMISSAO', 'NRVINCULOMSUBST', 'CDMUNICIPIO_END', 'NEWDTMESCOMPETENC'];

        foreach($arrayindex as $index){
            if(!isset($indexs[$index]) || empty($indexs[$index])){
                $indexs[$index] = null;
            }
        }
        return $indexs;
    }

    private function validaCpfExistePessoa($row, $validaPessoa){
        $cpf = $row['NRCPFPESSOA'];
        $nrorg = $row['NRORG'];
        $row['NRPESSOA'] = array_key_exists('NRPESSOA', $row) ? $row['NRPESSOA'] : null; 
        $pis = array_key_exists('NRPISPASEPPES', $row) ? $row['NRPISPASEPPES'] : null;
        $ctps = !empty($row['NRCTPSPESSOA']) ? $row['NRCTPSPESSOA'] : null;
        $serie = !empty($row['NRSERIECTPSPES']) ? $row['NRSERIECTPSPES'] : null;
        $sgctps = !empty($row['SGUFCTPSPES']) ? $row['SGUFCTPSPES'] : null;
        $rg = !empty($row['NRRGPESSOA']) ? $row['NRRGPESSOA'] : null;
        $pessoaCpf = null;
        $pessoaPis = null;
        $pessoaCtps = null;
        $pessoaRg = null;
        $message = '';
        
        $order = [".", "-"];
        $replace = '';
        $formatedCpf = str_replace($order, $replace, $cpf);
        $formatedPis = str_replace($order, $replace, $pis);
        $formatedRg = str_replace($order, $replace, $rg);
        
        if($cpf){
            $pessoaCpf = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array("nrcpfpessoa" => $formatedCpf, "nrorg" => $nrorg));
        }
        if($pis){
            $pessoaPis = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array("nrpispaseppes" => $formatedPis,"nrorg" => $nrorg));    
        }
        
        if(!empty($ctps) && !empty($serie) && !empty($sgctps)){
            $pessoaCtps = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array('sgufctpspes' => $sgctps,'nrseriectpspes' => $serie,"nrctpspessoa" => $ctps,"nrorg" => $nrorg));    
        } else if(!empty($ctps) && !empty($serie) && empty($sgctps)){
            $pessoaCtps = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array('nrseriectpspes' => $serie,"nrctpspessoa" => $ctps,"nrorg" => $nrorg));    
        } else if(!empty($ctps) && !empty($sgctps) && empty($serie)){
            $pessoaCtps = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array('sgufctpspes' => $sgctps,"nrctpspessoa" => $ctps,"nrorg" => $nrorg));    
        } else if(!empty($ctps)) {
            $pessoaCtps = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array("nrctpspessoa" => $ctps,"nrorg" => $nrorg)); 
        }
        
        if(!empty($rg)){
            $pessoaRg = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array("nrrgpessoa" => $formatedRg,"nrorg" => $nrorg));
            if(empty($pessoaRg)){
                $pessoaRg = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array("nrrgpessoa" => $rg,"nrorg" => $nrorg));
            }
        }
        
        if($pessoaCpf) {
            $nrpessoaCpf = $pessoaCpf->getNrpessoa();
            if(($nrpessoaCpf != $row['NRPESSOA'] || !$row['NRPESSOA']) && $validaPessoa){
                throw new \Exception(Messages::HCM00000_GER_PESSOA_CPF_ERROR.$pessoaCpf->getNmpessoa(), 10);
            }else if (($nrpessoaCpf != $row['NRPESSOA'] || !$row['NRPESSOA']) && !$validaPessoa){
                $message .= '- CPF já cadastrado para a pessoa: '.$pessoaCpf->getNmpessoa().' (Provável outro registro para mesmo CPF).<br>';
            }
        }
        if($pessoaPis) {
            $nrpessoaPis= $pessoaPis->getNrpessoa();
            if(($nrpessoaPis != $row['NRPESSOA'] || !$row['NRPESSOA']) && $validaPessoa){
                throw new \Exception(Messages::HCM00000_GER_PESSOA_PIS_ERROR.$pessoaPis->getNmpessoa(), 10);
            }else if (($nrpessoaPis != $row['NRPESSOA'] || !$row['NRPESSOA']) && !$validaPessoa){
                $message .= '- PIS já cadastrado para a pessoa: '.$pessoaPis->getNmpessoa().'(Provável outro registro para mesmo PIS).<br>';
            }    
        }
        if(!empty($pessoaCtps)) {
            $nrpessoaCtps= $pessoaCtps->getNrpessoa();
            if(($nrpessoaCtps != $row['NRPESSOA'] || !$row['NRPESSOA']) && $validaPessoa){
                throw new \Exception(Messages::HCM00000_GER_PESSOA_PIS_ERROR.$pessoaCtps->getNmpessoa(), 10);
            }else if (($nrpessoaCtps != $row['NRPESSOA'] || !$row['NRPESSOA']) && !$validaPessoa){
                $message .= '- CTPS já cadastrado para a pessoa: '.$pessoaCtps->getNmpessoa().'(Provável outro registro para mesmo CTPS).<br>';
            }    
        }
        if(!empty($pessoaRg)) {
            $nrpessoaRg= $pessoaRg->getNrpessoa();
            if(($nrpessoaRg != $row['NRPESSOA'] || !$row['NRPESSOA']) && $validaPessoa){
                throw new \Exception(Messages::HCM00000_GER_PESSOA_PIS_ERROR.$pessoaRg->getNmpessoa(), 10);
            }else if (($nrpessoaRg != $row['NRPESSOA'] || !$row['NRPESSOA']) && !$validaPessoa){
                $message .= '- RG já cadastrado para a pessoa: '.$pessoaRg->getNmpessoa().'(Provável outro registro para mesmo RG).<br>';
            }    
        }
        return $message; 
    }

    public function parcNegocioNovo($nrorg, $nmprincipalparc, $nmsecundariparc, $cdtipoparceiro, $idpessoafisica, $idinstituicao,
                                            $cdtipoinscricao, $nrinscricaoparc, $dtnascifundparc) {
        $idativo = 'S';
        $parceiro = new $this->parcnegocio;
        $parceiro->setNrorg($nrorg);
        $parceiro->setIdativo($idativo);
        $parceiro->setNmprincipalparc($nmprincipalparc);
        $parceiro->setNmsecundariparc($nmsecundariparc);
        $parceiro->setCdtipoparcprincipal($cdtipoparceiro);
        $parceiro->setIdpessoafisica($idpessoafisica);
        $parceiro->setIdinstituicao($idinstituicao);
        $parceiro->setCdtipoinscricao($cdtipoinscricao);
        $parceiro->setNrinscricaoparc($nrinscricaoparc);
        $parceiro->setDtnascifundparc($dtnascifundparc);

        $parceiro->setDtinclusao(DateUtil::getDataAtual());
        $parceiro->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $parceiro->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $parceiro->setDtultatu(DateUtil::getDataAtual());
        $parceiro->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $parceiro->setCdoperultatu($this->hcmParameters->getCdoperador());

        $parceiro->setNrparcnegocio(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'PARCNEGOCIO', 1, 12, true));
        
        $this->entityManager->persist($parceiro);
        $this->entityManager->flush($parceiro); 

        return $parceiro;
    }

    public function pessoaNova(
        $nrorg, $nrparcnegocio, $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento,
        $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS,
        $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento,
        $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao,
        $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $nrCategoriaMilitar,
        $nrCertificadoReservista, $dsCertificadoReservista, $dtCertificadoReservista,
        $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado,
        $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG, $dsLocalExpedicaoRG, $dtExpedicaoRG,
        $nrNacionalidade, $nrGrauInstrucao, $pronometrat = null, $nranochegapais = null,
        $nrdocuestrang = null, $idpesnaturaliza = null, $dtnaturalizapes = null, $idposvistoperm = null,
        $dtpermavisto = null, $nrconselhoreg = null, $nrinscrconsreg = null, $dtexconsreg = null,
        $sgufconsreg = null, $cdtpbaixapag = null, $nrlivrocertnasc = null, $nrfolhacertnasc = null, $dscomplcertnasc = null,$nrregctpspes = null,
        $nrclassestrang = null, $dsexpdocestr = null, $dtchegadapais = null, $idcotapcd = null
    )
    {
        $pessoa = $this->novoPessoaMestre($nrorg, $nrparcnegocio);
        $pessoah = $this->novoPessoaHistorico(
            $nrorg, $pessoa->getNrpessoa(), $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento,
            $dtNascimento, $idSexo, $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS,
            $dtPIS, $nrInscricaoINSS, $nrInscricaoISS, $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento,
            $idIsentoTituloEleitor, $nrTituloEleitor, $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao,
            $dtHabilitacao, $dtValidadeHabilitacao, $categoriaHabilitacao, $nrCategoriaMilitar, $nrCertificadoReservista,
            $dsCertificadoReservista, $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca,
            $idGrupoSanguineo, $idFatorRH, $cdpais, $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG,
            $dsLocalExpedicaoRG, $dtExpedicaoRG, $nrNacionalidade, $nrGrauInstrucao, $pronometrat, $nranochegapais, $nrdocuestrang,
            $idpesnaturaliza, $dtnaturalizapes, $idposvistoperm, $dtpermavisto, $nrconselhoreg, $nrinscrconsreg, $dtexconsreg,
            $sgufconsreg, $cdtpbaixapag, $nrlivrocertnasc, $nrfolhacertnasc, $dscomplcertnasc,$nrregctpspes,
            $nrclassestrang, $dsexpdocestr, $dtchegadapais, $idcotapcd 
        );
        return [$pessoa, $pessoah];
    }

    private function novoPessoaMestre($nrorg, $nrparcnegocio) {
        $pessoa = new $this->gpePessoa;

        $pessoa->setNrorg($nrorg);
        $pessoa->setNrparcnegocio($nrparcnegocio);

        $pessoa->setDtinclusao(DateUtil::getDataAtual());
        $pessoa->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoa->setDtultatu(DateUtil::getDataAtual());
        $pessoa->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoa->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoa->setNrpessoa(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOA', 1, 12, true));

        $this->entityManager->persist($pessoa);
        $this->entityManager->flush($pessoa);

        return $pessoa;
    }

    private function novoPessoaHistorico(
        $nrorg, $nrpessoa, $dtmescompetencia, $nmpessoa, $apelido, $nrCertidaoNascimento, $dtNascimento, $idSexo,
        $nrCondicaoFisica, $nrCPF, $nrCTPS, $nrSerieCTPS, $sgufCTPS, $dtCTPS, $nrPIS, $dtPIS, $nrInscricaoINSS,
        $nrInscricaoISS, $cdEstadoCivil, $nrCertidaoCasamento, $dtCasamento, $idIsentoTituloEleitor, $nrTituloEleitor,
        $nrSecaoEleitoral, $nrZonaEleitoral, $nrHabilitacao, $dtHabilitacao, $dtValidadeHabilitacao,
        $categoriaHabilitacao, $nrCategoriaMilitar, $nrCertificadoReservista, $dsCertificadoReservista,
        $dtCertificadoReservista, $cdExpedicaoCertificadoReservista, $nrRaca, $idGrupoSanguineo, $idFatorRH, $cdpais,
        $sgEstado, $cdMunicipio, $nrRg, $orgaoExpedidorRG, $sgExpedidorRG, $dsLocalExpedicaoRG, $dtExpedicaoRG,
        $nrNacionalidade, $nrGrauInstrucao, $pronometrat = null,  $nranochegapais = null,
        $nrdocuestrang = null, $idpesnaturaliza = null, $dtnaturalizapes = null, $idposvistoperm = null,
        $dtpermavisto = null, $nrconselhoreg = null, $nrinscrconsreg = null, $dtexconsreg = null,
        $sgufconsreg = null, $cdtpbaixapag = null, $nrlivrocertnasc = null, $nrfolhacertnasc = null, $dscomplcertnasc = null,$nrregctpspes = null,
        $nrclassestrang = null, $dsexpdocestr = null, $dtchegadapais = null, $idcotapcd = null
    )
    {
        $pessoah = new $this->gpePessoah;

        //setDadosPrincipais
        list($primeiroNome, $meioNome, $ultimoNome) = self::decompoeNomePessoa($nmpessoa);

        $pessoah->setNrorg($nrorg);
        $pessoah->setNrpessoa($nrpessoa);
        $pessoah->setNmpessoa($nmpessoa);
        $pessoah->setDsprinomepess($primeiroNome);
        $pessoah->setDsnomemeiopess((strlen($meioNome)> 20) ? substr($meioNome, 0, 20) : $meioNome);
        $pessoah->setDsultnomepess($ultimoNome);
        $pessoah->setDsapelidopess($apelido);
        $pessoah->setDtmescompetenc($dtmescompetencia);
        $pessoah->setDtnascpessoa($dtNascimento);
        $pessoah->setNrcertnascpes($nrCertidaoNascimento);
        $pessoah->setNrlivrocertnasc($nrlivrocertnasc);
        $pessoah->setNrfolhacertnasc($nrfolhacertnasc);
        $pessoah->setDscomplcertnasc($dscomplcertnasc);

        //setRegistroIdentidade
        $pessoah->setNrcpfpessoa($nrCPF);
        $pessoah->setNrrgpessoa($nrRg);
        $pessoah->setCdexrgpessoa($orgaoExpedidorRG);
        $pessoah->setSgufrgpessoa($sgExpedidorRG);
        $pessoah->setDslocalexrgpes($dsLocalExpedicaoRG);
        $pessoah->setDtexrgpessoa($dtExpedicaoRG);

        //setLocal
        $pessoah->setNrnacionalid($nrNacionalidade);
        $pessoah->setCdpais($cdpais);
        $pessoah->setSgestado($sgEstado);
        $pessoah->setCdmunicipio($cdMunicipio);

        //setImigrante
        $pessoah->setNranochegapais($nranochegapais);
        $pessoah->setNrdocuestrang($nrdocuestrang);
        $pessoah->setIdpesnaturaliza($idpesnaturaliza);
        $pessoah->setDtnaturalizapes($dtnaturalizapes);
        $pessoah->setIdposvistoperm($idposvistoperm);
        $pessoah->setDtpermavisto($dtpermavisto);
        $pessoah->setNrclassestrang($nrclassestrang);
        $pessoah->setDsexpdocestr($dsexpdocestr);
        $pessoah->setDtchegadapais($dtchegadapais ? DateUtil::getDataDeString($dtchegadapais) : null);

        //setDadosCarteiraTrabalho
        $pessoah->setNrctpspessoa($nrCTPS);
        $pessoah->setNrseriectpspes($nrSerieCTPS);
        $pessoah->setSgufctpspes($sgufCTPS);
        $pessoah->setDtctpspessoa($dtCTPS);
        $pessoah->setNrregctpspes($nrregctpspes);

        //setDadosImpostos
        $pessoah->setNrpispaseppes($nrPIS);
        $pessoah->setDtinscpispasep($dtPIS);
        $pessoah->setNrinscinsspes($nrInscricaoINSS);
        $pessoah->setNrinscisspes($nrInscricaoISS);

        //setDadosCivis
        $pessoah->setCdestacivil($cdEstadoCivil);
        $pessoah->setNrcertcasapes($nrCertidaoCasamento);
        $pessoah->setDtcasapessoa($dtCasamento);

        //setDadosEleitorais
        $pessoah->setIdisentituelei($idIsentoTituloEleitor);
        $pessoah->setNrtitueleipes($nrTituloEleitor);
        $pessoah->setNrsecaeleipes($nrSecaoEleitoral);
        $pessoah->setNrzonaeleipes($nrZonaEleitoral);

        //setDadosHabilitacaoVeiculo
        $pessoah->setNrcarthabpes($nrHabilitacao);
        $pessoah->setDthabcartpes($dtHabilitacao);
        $pessoah->setDtvalhabcartpes($dtValidadeHabilitacao);
        $pessoah->setDscateghabcart($categoriaHabilitacao);

        //setDadosMilitares
        $pessoah->setNrcatmilitarpe($nrCategoriaMilitar);
        $pessoah->setNrcertresepes($nrCertificadoReservista);
        $pessoah->setDscategcertrese($dsCertificadoReservista);
        $pessoah->setDtcertresepes($dtCertificadoReservista);
        $pessoah->setCdexcertresepes($cdExpedicaoCertificadoReservista);

        //setDadosFisicos
        $pessoah->setIdsexopessoa($idSexo);
        $pessoah->setNrracapessoa($nrRaca);
        $pessoah->setNrcondfispes($nrCondicaoFisica);
        $pessoah->setIdgrupsangupes($idGrupoSanguineo);
        $pessoah->setIdcotapcd($idcotapcd);
        $pessoah->setIdfatorrhpes($idFatorRH);

        //setDadosAdicionais
        $pessoah->setNrgrauinstr($nrGrauInstrucao);
        $pessoah->setNrconselhoreg($nrconselhoreg);
        $pessoah->setNrinscrconsreg($nrinscrconsreg);
        $pessoah->setDtexconsreg($dtexconsreg);
        $pessoah->setSgufconsreg($sgufconsreg);
        $pessoah->setCdtpbaixapag($cdtpbaixapag);

        if($pronometrat) {
            $pessoah->setDspronometrat($pronometrat);
        }else{
            $pronomeTratamento = $this->getPronomeTratamento($cdEstadoCivil, $idSexo);
            $pessoah->setDspronometrat($pronomeTratamento);
        }

        $pessoah->setDtinclusao(DateUtil::getDataAtual());
        $pessoah->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $pessoah->setDtultatu(DateUtil::getDataAtual());
        $pessoah->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $pessoah->setCdoperultatu($this->hcmParameters->getCdoperador());

        $pessoah->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_PESSOAH', 1, 12, true));
        
        $this->entityManager->persist($pessoah);
        $this->entityManager->flush($pessoah);

        return $pessoah;
    }

    private function decompoeNomePessoa($nmpessoa) {
        $arrayNmpessoa = explode(' ', $nmpessoa);
        $nomeMeio      = '';

        $primeiroNome = $arrayNmpessoa[0];
        for ($i = 1; $i < count($arrayNmpessoa) - 1; $i++) {
            $nomeMeio .= ' ' . $arrayNmpessoa[$i];
            $nomeMeio = trim($nomeMeio);
        }
        $ultimoNome = $arrayNmpessoa[count($arrayNmpessoa) - 1];
        $nomeMeio   = ($nomeMeio) ? : $ultimoNome;

        return [$primeiroNome, $nomeMeio, $ultimoNome];
    }

    private function getPronomeTratamento($cdEstadoCivil, $idSexoPessoa) {
        if ($idSexoPessoa == 'M'){
            $pronomeTratamento = 'Sr.';
            return $pronomeTratamento;
        }
        else if ($idSexoPessoa == 'F' && $cdEstadoCivil == 'C'){
            $pronomeTratamento = 'Sra.';
            return $pronomeTratamento;
        }
        else {
            $pronomeTratamento = 'Srta.';
            return $pronomeTratamento;
        }
    }

    private function existeFormacaoAcademicaParaSerAdicionado($pessoa) {
        if(!empty(($pessoa['DES_AREAFORMACAO_NRORG'])) || !empty(($pessoa['DSANDAMENTOCURSO'])) || !empty(($pessoa['DSCURSO'])) || !empty(($pessoa['DSINSTITUICAO']))
        || !empty(($pessoa['NMINSTITUICAOPARC'])) || !empty(($pessoa['NRINSTITUICAOPARC'])) || !empty(($pessoa['NRAREAFORMACAO'])) || !empty(($pessoa['CDIDIOMA'])) || !empty(($pessoa['IDNVIDIOMA'])) || !empty(($pessoa['DSOBSERVACAO']))
        || !empty(($pessoa['NRCARGAHORARIA'])) || !empty(($pessoa['NRCURSO'])) || !empty(($pessoa['NRTIPOFORMACAO'])) || !empty(($pessoa['DTFIM'])) || !empty(($pessoa['DTINICIO']))) {
            return true;
        }
    }
    
    private function novoFormacaoAcademica($parcnegocio, $pessoa) {
        if($this->existeFormacaoAcademicaParaSerAdicionado($pessoa) == true) {

            $formacaoAcad = new $this->formacaoAcad;
            $formacaoAcad->setNrorg($this->hcmParameters->getNrorgTrabalho());
            $formacaoAcad->setDtinclusao(DateUtil::getDataAtual());
            $formacaoAcad->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $formacaoAcad->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $formacaoAcad->setDtultatu(DateUtil::getDataAtual());
            $formacaoAcad->setCdoperultatu($this->hcmParameters->getCdoperador());
            $formacaoAcad->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $formacaoAcad->setNrparcnegocio($parcnegocio);
            $formacaoAcad->setDesAreaformacaoNrorg(($pessoa['DES_AREAFORMACAO_NRORG']));
            $formacaoAcad->setDsandamentocurso(($pessoa['DSANDAMENTOCURSO']));
            $formacaoAcad->setDscurso(isset($pessoa['DSCURSO']) ? ($pessoa['DSCURSO']) : null);
            $formacaoAcad->setDsinstituicao(($pessoa['DSINSTITUICAO']));
            $formacaoAcad->setNrinstituicaoparc(isset($pessoa['NRINSTITUICAOPARC']) ? ($pessoa['NRINSTITUICAOPARC']) : null);
            $formacaoAcad->setNrareaformacao(isset($pessoa['NRAREAFORMACAO']) ? ($pessoa['NRAREAFORMACAO']) : null);
            $formacaoAcad->setCdidioma(isset($pessoa['CDIDIOMA']) ? ($pessoa['CDIDIOMA']) : null);
            $formacaoAcad->setIdnvidioma(isset($pessoa['IDNVIDIOMA']) ? ($pessoa['IDNVIDIOMA']) : null);
            $formacaoAcad->setDsobservacao(($pessoa['DSOBSERVACAO']));
            $formacaoAcad->setDtfim(DateUtil::getDataDeString(substr(($pessoa['DTFIM']),0,10),DateUtil::FORMATO_BRASILEIRO,true));
            $formacaoAcad->setDtinicio(DateUtil::getDataDeString(substr(($pessoa['DTINICIO']),0,10),DateUtil::FORMATO_BRASILEIRO,true));
            $formacaoAcad->setNrcargahoraria(($pessoa['NRCARGAHORARIA']));
            $formacaoAcad->setNrcurso(!empty($pessoa['NRCURSO']) ? ($pessoa['NRCURSO']) : null);
            $formacaoAcad->setNrtipoformacao(isset($pessoa['NRTIPOFORMACAO']) ? ($pessoa['NRTIPOFORMACAO']) : null);
            $formacaoAcad->setNrformacao(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'DES_FORMACAO', 1, 10, true));
            
            $this->entityManager->persist($formacaoAcad);
            $this->entityManager->flush($formacaoAcad);
        }
    }

    private function convertDsLogrToCdlogr($dsLogr){
        $retorno = null;
        
        if(is_string($dsLogr)) $dsLogr = strtoupper($dsLogr);
        
        // Mapa dslogradouro=>cdlogradouro.
        $map = [
            "RUA" => "RUA",
            "AVENIDA" => "AV",
            "PRAÇA" => "PCA",
            "PRACA" => "PCA",
            "ALAMEDA" => "AL",
            "ESTRADA" => "EST",
            "RODOVIA" => "RDV",
            "QUADRA" => "QD",
            "AEROPORTO" => "AER",
            "APARTAMENTO" => "AP",
            "BECO" => "BC",
            "BLOCO" => "BL",
            "CAMINHO" => "CAM",
            "ESCADINHA" => "ESCD",
            "FAZENDA" => "FAZ",
            "FORTALEZA" => "FORT",
            "GALERIA" => "GL",
            "QUILOMETRO" => "KM",
            "LADEIRA" => "LD",
            "LARGO" => "LGO",
            "PRAIA" => "PR",
            "QUINTA" => "QTA",
            "TRAVESSA" => "TRV",
            "VIADUTO" => "VD",
            "VILA" => "VL",
            "SUPER QUADRA" => "SQD",
            "PARQUE" => "PRQ",
            "CHÁCARA" => "CH",
            "CHACARA" => "CH",
            "CALÇADA" => "CALC",
            "CALCADA" => "CALC",
            "VIA DE ACESSO"=>"V-AC",
            "VIA" => "VIA",
            "SÍTIO" => "SIT",
            "SITIO" => "SIT",
            "ACESSO" => "AC",
            "LOTEAMENTO" => "LOT",
            "PASSAGEM" => "PSG",
            "VIELA" => "VLA",
            "FAVELA" => "FAV",
            "FEIRA" => "FRA",
            "JARDIM" => "JD",
            "LAGO" => "LG",
            "LAGOA" => "LGA",
            "MORRO" => "MRO",
            "TRECHO" => "TR",
            "VALE" => "VLE",
            "VEREDA" => "VER",
            "ACAMPANHAMENTO" => "0003",
            "AREA" => "A",
            "ÁREA" => "A",
            "CAMPO" => "CPO",
            "COLÔNIA" => "COL",
            "COLONIA" => "COL",
            "CONDOMÍNIO" => "CON",
            "CONDOMINIO" => "CON",
            "CONJUNTO" => "CJ",
            "DISTRITO" => "DT",
            "ESPLANADA" => "ESP",
            "NÚCLEO" => "NUC",
            "NUCLEO" => "NUC",
            "OUTROS" => "O",
            "PASSARELA" => "PSA",
            "PÁTIO" => "PAT",
            "PATIO" => "PAT",
            "RECANTO" => "REC",
            "RESIDENCIAL" => "RES",
            "SETOR" => "ROD",
            "ACAMPAMENTO" => "0004",
        ];
        
        if(isset($map[$dsLogr]) && !empty($map[$dsLogr]))
            $retorno = $map[$dsLogr];
        
        return $retorno;
    }

    private function novoEndereco($nrparcnegocio, $cdpais, $sgestado, $cdmunicipio, $cdlogradouro, $nmbairroendereco,
                         $cdtipoendereco, $nrcependereco, $dscompleendereco, $nrimovelendereco, $dsendereco,
                         $dsreferenciaende)
    {
        $endereco = new $this->enderecoParc;

        $endereco->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $endereco->setDtinclusao(DateUtil::getDataAtual());
        $endereco->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $endereco->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $endereco->setDtultatu(DateUtil::getDataAtual());
        $endereco->setCdoperultatu($this->hcmParameters->getCdoperador());
        $endereco->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $endereco->setNrparcnegocio($nrparcnegocio);
        $endereco->setCdpais($cdpais);
        $endereco->setIdativo('S');
        $endereco->setSgestado($sgestado);
        $endereco->setCdmunicipio($cdmunicipio);
        $endereco->setCdlogradouro($cdlogradouro);
        $endereco->setNmbairroendereco($nmbairroendereco);
        $endereco->setCdtipoendereco($cdtipoendereco);
        $endereco->setNrcependereco($nrcependereco);
        $endereco->setDscompleendereco($dscompleendereco);
        $endereco->setNrimovelendereco($nrimovelendereco);
        $endereco->setDsendereco($dsendereco);
        $endereco->setDsreferenciaende($dsreferenciaende);
        $endereco->setNrenderecoparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ENDERECOPARC', 1, 10, true));
        
        $this->entityManager->persist($endereco);
    }

    private function novoContaCorrente($nrparcnegocio, $cdreducont, $cdbanco, $cdagencia, $cdcontcorr, $nrtpcontcorr) {
        $contaCorr = new $this->contCorrParc;
        $contaCorr->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $contaCorr->setDtinclusao(DateUtil::getDataAtual());
        $contaCorr->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $contaCorr->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $contaCorr->setDtultatu(DateUtil::getDataAtual());
        $contaCorr->setCdoperultatu($this->hcmParameters->getCdoperador());
        $contaCorr->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $contaCorr->setIdativo('S');
        $contaCorr->setNrparcnegocio($nrparcnegocio);
        $contaCorr->setCdreducont($cdreducont);
        $contaCorr->setCdbanco($cdbanco);
        $contaCorr->setCdagencia($cdagencia);
        $contaCorr->setCdcontcorr($cdcontcorr);
        $contaCorr->setNrtpcontcorr($nrtpcontcorr);
        $contaCorr->setNrcontcorrparc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'CONTCORRPARC', 1, 10, true));
        $this->entityManager->persist($contaCorr);
    }
    
    private function novaFormacomu($nrparcnegocio, $cdformacomu, $cdprefixcomuparc, $cdcomunicaparc, $cdcomplecomuparc, $nrenderecoparc) {
        $obj = new $this->comunicaParc;
        $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtinclusao(DateUtil::getDataAtual());
        $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtultatu(DateUtil::getDataAtual());
        $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
        $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $obj->setIdativo('S');
        $obj->setNrparcnegocio($nrparcnegocio);
        $obj->setCdformacomu($cdformacomu);
        $obj->setCdprefixcomuparc($cdprefixcomuparc);
        $obj->setCdcomunicaparc($cdcomunicaparc);
        $obj->setCdcomplecomuparc($cdcomplecomuparc);
        $obj->setNrenderecoparc($nrenderecoparc);
        $nrcomunicaparc = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'COMUNICAPARC', 1, 12, true);
        $obj->setNrcomunicaparc($nrcomunicaparc);
        
        $this->entityManager->persist($obj);
        $this->entityManager->flush($obj);
    }

    private function operadorNovo($nrorgtrab,$nmoperador,$dtnascimento,$email,$nrparcnegocio,$senha,$idoperativo = null,$idstatusenha = null,
                                 $idatuoperador = null,$idcadpraprovore = null,$cdfunc = null,
                                 $idcadprproceore = null,$idligextoper = null,
                                 $idsenhvardia = null,$idopermaster = null,$idoperacesso = null,$nmacesso = null)
    {
        $operador = new $this->operador;
        $operador->setNrorg($nrorgtrab);
        $operador->setIdoperativo($idoperativo);
        $operador->setIdstatusenha($idstatusenha);
        $operador->setIdatuoperador($idatuoperador);
        $operador->setIdcadpraprovore($idcadpraprovore);
        $operador->setDtultatuoper(DateUtil::getDataAtual());
        $operador->setNrorgtrab($nrorgtrab);
        $operador->setCdfunc($cdfunc);
        $operador->setDtnascimento($dtnascimento);
        $operador->setNmacesso(is_null($nmacesso) ? $email : $nmacesso);
        $operador->setDsemailoper($email);
        $operador->setIdcadprproceore($idcadprproceore);
        $operador->setIdligextoper($idligextoper);
        $operador->setIdsenhvardia($idsenhvardia);
        $operador->setIdopermaster($idopermaster);
        $operador->setIdoperacesso($idoperacesso);
        $operador->setNrparcnegocio($nrparcnegocio);
        $operador->setNmoperador($nmoperador);
        $operador->setCdsenhaoperweb($senha);
        $operador->setDtinclusao(DateUtil::getDataAtual());
        $operador->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $operador->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $operador->setDtultatu(DateUtil::getDataAtual());
        $operador->setCdoperultatu($this->hcmParameters->getCdoperador());
        $operador->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $operador->setCdoperador(str_pad((string)$this->getNextId(), 12, "0", STR_PAD_LEFT));

        $this->entityManager->persist($operador);
        $this->entityManager->flush($operador);

        return $operador;
    }

    private function getNextId() {
        $idRow = $this->importacaoPlanilhaVinculoRepository->getMaxId();
        $id = $idRow[0]['MAX_ID'];
        return $id;
    }
    
}