<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2500Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2500($params) {
        try {
            $sql = <<<SQL
/*getItensParaEnvioS2500*/
SELECT DISTINCT VM.NRPESSOA, VM.NRPESSOA as NRCHAVE,
       '1' as NRTABELA,
       'I' as IDOPERACAO,
       PR.DTPERPGTO AS DTMESCOMPETENC
  FROM GPE_PROCESSTRAB PR
  JOIN GPE_INFOPROCTRAB IP ON IP.NRORG = PR.NRORG
                          AND IP.NRPROCESSTRAB = PR.NRPROCESSTRAB
  JOIN GPE_VINCULOM VM ON VM.NRORG = IP.NRORG
                      AND VM.NRVINCULOM = IP.NRVINCULOM
 WHERE PR.NRORG = :P_NRORG
   AND PR.NRESTRUTURAM = :P_NRESTRUTURAM
   AND PR.DTPERPGTO = :P_DTMESCOMPETENC
   AND (VM.NRPESSOA = :P_NRPESSOA OR :P_NRPESSOA IS NULL)
 ORDER BY 1
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
                "P_NRPESSOA" => isset($params['NRPESSOA']) ? $params['NRPESSOA'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosS2500($params) {
        try {
            $dados = $this->getDadosPrincipais($params['NRENVIO'], $params['NRCHAVE'],  $params['NRESTRUTURAM'],  $params['NRTPAMBIENTE'],  $params['CDCNPJEMPREGADOR']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $cdcnpjempregador) {
        $sql = <<<SQL
/*getDadosS2500*/
SELECT PR.NRPROCESSTRAB CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --ideResp
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') ideResp_tpInsc,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END ideResp_nrInsc,
       --infoProcesso
       PR.NRTPPROCESSTRAB origem,
       REPLACE(REPLACE(PR.CDPROCESSTRAB,'.',''),'-','') nrProcTrab,
       SUBSTR(PR.DSOBSERVACAO,0,999) obsProcTrab,
       --dadosCompl
       --infoProcJud
       DECODE(PR.NRTPPROCESSTRAB,1,TO_CHAR(PR.DTSETENCA, 'YYYY-MM-DD'),'') dtSent,
       DECODE(PR.NRTPPROCESSTRAB,1,PR.SGUFVARAPROTRAB,'') ufVara,
       DECODE(PR.NRTPPROCESSTRAB,1,MU.CDMUNICIBGE,'') codMunic,
       DECODE(PR.NRTPPROCESSTRAB,1,PR.CDVARAPROCETRAB,'') idVara,
       --infoCCP
       DECODE(PR.NRTPPROCESSTRAB,2,TO_CHAR(PR.DTCCP, 'YYYY-MM-DD'),'') dtCCP,
       DECODE(PR.NRTPPROCESSTRAB,2,PR.NRTPCCP,'') tpCCP,
       DECODE(PR.NRTPPROCESSTRAB,2,REPLACE(REPLACE(REPLACE(PR.CDCNPJCCP,'.',''),'/',''),'-',''),'') cnpjCCP,
       --ideTrab
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.',''),'-','') cpfTrab,
       PH.NMPESSOA nmTrab,
       TO_CHAR(PH.DTNASCPESSOA, 'YYYY-MM-DD') dtNascto,
       PR.NRPROCESSTRAB, PR.NRPARCNEGOCIO, VM.NRPESSOA,
       :P_NRTPAMBIENTE NRTPAMBIENTE, :P_CDCNPJEMPREGADOR CDCNPJEMPREGADOR
  FROM GPE_PROCESSTRAB PR
  JOIN GPE_INFOPROCTRAB IP ON IP.NRORG = PR.NRORG
                          AND IP.NRPROCESSTRAB = PR.NRPROCESSTRAB
  JOIN GPE_VINCULOM VM ON VM.NRORG = IP.NRORG
                      AND VM.NRVINCULOM = IP.NRVINCULOM
  JOIN GPE_PESSOAH PH ON PH.NRORG = VM.NRORG
                     AND PH.NRPESSOA = VM.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG
                                                 AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Responsavel Direto*/
  LEFT JOIN PARCNEGOCIO PN ON PN.NRORG = PR.NRORG
                          AND PN.NRPARCNEGOCIO = PR.NRPARCNEGOCIO
  /*Vara*/
  LEFT JOIN MUNICIPIO MU ON MU.CDMUNICIPIO = PR.CDMUNIVARPRTRAB
                        AND (MU.NRORG = PR.NRORG OR MU.NRORG = 0)
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE PR.NRORG = :P_NRORG
   AND VM.NRPESSOA = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_CDCNPJEMPREGADOR" => $cdcnpjempregador,
        );
        return $this->connection->fetchAssoc($sql, $params);
    
    }
    
    public function getDadosS2500infoContr($params){
        try{
            $dados = $this->getDadosinfoContr(
                $params['NRORG'], $params['NRPROCESSTRAB'], $params['NRPARCNEGOCIO'], $params['NRPESSOA'], $params['NRTPAMBIENTE'], $params['CDCNPJEMPREGADOR']
            );
            foreach ($dados as &$dado) {
                if ($dado['REPERCPROC'] == '1') {
                    $valores = $this->getValoresInfoVlr(
                        $dado['NRORG'], $dado['NRINFOPROCTRAB'], $dado['DTMESCOMPINI'], $dado['DTMESCOMPFIM'], $dado['NRVINCULOM'], $dado['CDESOCIAL'], $dado['NRAGENTENOCIVO']
                    );
                    if (!empty($valores)) {
                        $dado['VRREMUN'] = $valores['VRREMUN'];
                    }
                }
            }
            return $dados;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosinfoContr($nrorg, $nrprocesstrab, $nrparcnegocio, $nrpessoa, $nrtpambiente, $cdcnpjempregador) {
        $sql = <<<SQL
/*getDadosinfoContr*/
SELECT IP.NRTPCONTRPROC tpContr,
       DECODE(:P_NRPARCNEGOCIO,NULL,DECODE(ENVIO.NRENVIO,NULL,'N','S'),'N') indContr,
       CASE WHEN IP.NRTPCONTRPROC IN (2,4) THEN TO_CHAR(VM.DTADMISSAOVINC, 'YYYY-MM-DD') ELSE NULL END dtAdmOrig,
       IP.IDREINT indReint,
       IP.IDCATEG indCateg,
       IP.IDNATATIV indNatAtiv,
       IP.IDMOTDESLIG indMotDeslig,
       NVL(VH.CDMATRICULAESOCIAL,VM.NRVINCULOM) matricula,
       '' codCateg,
       DECODE(:P_NRPARCNEGOCIO,NULL,DECODE(ENVIO.NRENVIO,NULL,DECODE(IDSIMPLIFICADO,1,VM.DTADMISSAOVINC,NULL),NULL),DECODE(IDSIMPLIFICADO,1,VM.DTADMISSAOVINC,NULL)) dtInicio,
       --infoCompl
       --remuneracao
       --infoVinc
       ----duracao
       ----observacoes
       ----sucessaoVinc
       ----infoDeslig
       --infoTerm
       --mudCategAtiv
       --unicContr
       --ideEstab
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') ideEstab_tpInsc,
       PN.NRINSCRICAOPARC ideEstab_nrInsc,
       --infoVlr
       TO_CHAR(IP.DTMESCOMPINI, 'YYYY-MM') compIni,
       TO_CHAR(IP.DTMESCOMPFIM, 'YYYY-MM') compFim,
       IP.NRREPERC indReperc,
       DECODE(IP.NRREPERC,1,1,2,0,NULL) vrremun,
       0 vrapi,
       0 vr13api,
       0 vrinden,
       '' vrbaseindenfgts,
       '' pagDiretoResc,
       DECODE(IP.IDINDENSD,'S','S','') indenSD,
       DECODE(IP.IDINDENABONO,'S','S','') indenAbono,
       IP.NRORG, IP.NRINFOPROCTRAB, IP.DTMESCOMPINI, IP.DTMESCOMPFIM, IP.NRVINCULOM, CE.CDESOCIAL, VH.NRAGENTENOCIVO
  FROM GPE_INFOPROCTRAB IP
  JOIN GPE_VINCULOM VM ON VM.NRORG = IP.NRORG
                      AND VM.NRVINCULOM = IP.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  LEFT JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ET ON ET.NRORG = VH.NRORG
                    AND ET.NRESTRUTURAM = VH.NRESTRUTLEGAL
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                     AND PN.NRORG = ET.NRORG
  /*Envio de admissao*/
  LEFT JOIN (SELECT NRORG, NRVINCULOM, MAX(NRENVIO) NRENVIO, MAX(IDSIMPLIFICADO) IDSIMPLIFICADO
              FROM (SELECT EC.NRORG, EC.NRVINCULOM, EC.NRENVIO, 0 IDSIMPLIFICADO
                     FROM ESO_ENVIOCADASTROS EC
                     JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                       AND EV.NMEVENTO IN ('S-2190','S-2200','S-2300')
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = :P_NRORG
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                    UNION ALL
                    SELECT EC.NRORG, EC.NRVINCULOM, EC.NRENVIO, 1 IDSIMPLIFICADO
                     FROM ESO_NAOPERIODICO EC
                     /*VERIFICA EMPRESAS DO MESMO GRUPO - CNPJ RAIZ*/
                     JOIN ESTRUTURAH EVH ON EVH.NRORG = EC.NRORG
                                        AND EVH.NRESTRUTURAM = EC.NRESTRUTURAM
                                        AND SUBSTR(REPLACE(REPLACE(REPLACE(EVH.CDCNPJESTRUT,'.'),'/'),'-'),1,8) = SUBSTR(REPLACE(REPLACE(REPLACE(:P_CDCNPJEMPREGADOR,'.'),'/'),'-'),1,8)
                                        AND EVH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                                   WHERE H1.NRORG = EVH.NRORG
                                                                     AND H1.NRESTRUTURAM = EVH.NRESTRUTURAM
                                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                    WHERE EC.NRORG = :P_NRORG
                      AND EC.NMEVENTO IN ('S-2190','S-2200','S-2300')
                      AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                      AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                      AND EC.NRENVIOEXCL IS NULL /*QUE NAO FOI EXCLUIDO*/
                      AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE)
            GROUP BY NRORG, NRVINCULOM) ENVIO ON ENVIO.NRORG = VM.NRORG
                                             AND ENVIO.NRVINCULOM = VM.NRVINCULOM
 WHERE IP.NRORG = :P_NRORG
   AND IP.NRPROCESSTRAB = :P_NRPROCESSTRAB
   AND VM.NRPESSOA = :P_NRPESSOA
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRPROCESSTRAB" => $nrprocesstrab,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
            "P_NRPESSOA" => $nrpessoa,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_CDCNPJEMPREGADOR" => $cdcnpjempregador,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getValoresInfoVlr($nrorg, $nrinfoproctrab, $dtmescompini, $dtmescompfim, $nrvinculom, $codcateg, $nragentenocivo){
        try{
            $sql = <<<SQL
/*getValoresInfoVlr*/
SELECT SUM(CASE WHEN IC.NREVENTOM IN (4500,4505,4510,4515,4520,4525) THEN IC.VREVENTOFOLHA ELSE 0 END) vrRemun,
       0 vrAPI,
       0 vr13API,
       0 vrInden,
       0 vrBaseIndenFGTS
  FROM FPA_ITECALCFOLHA IC
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IC.NRORG
                          AND CF.NRCALCULOFOLHA = IC.NRCALCULOFOLHA
                          AND CF.NRTIPOMOVIMENT = 100 /*PROCESSO TRABALHISTA*/
                          AND CF.NRTPMODALIDCAL = 1
                          AND CF.DTOCORRENCIA BETWEEN :P_DTMESCOMPINI AND :P_DTMESCOMPFIM
 WHERE IC.NRORG = :P_NRORG
   AND IC.NRVINCULOM = :P_NRVINCULOM
   AND IC.NREVENTOM IN (4500,4505,4510,4515,4520,4525)
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPINI" => $dtmescompini,
                "P_DTMESCOMPFIM" => $dtmescompfim,
                "P_NRVINCULOM" => $nrvinculom,
            );
            return $this->connection->fetchAssoc($sql, $params);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosS2500idePeriodo($params){
        try{
            $dados = $this->getDadosidePeriodo(
                $params['NRORG'], $params['NRINFOPROCTRAB'], $params['DTMESCOMPINI'], $params['DTMESCOMPFIM'], $params['NRVINCULOM'], $params['CDESOCIAL'], $params['NRAGENTENOCIVO']
            );
            
            foreach ($dados as &$dado) {
                if ($dado['VRBCFGTSPROCTRAB'] + $dado['VRBCFGTSSEFIP'] + $dado['VRBCFGTSDECANT'] == 0) {
                    $dado['VRBCFGTSPROCTRAB'] = null;
                    $dado['VRBCFGTSSEFIP'] = null;
                    $dado['VRBCFGTSDECANT'] = null;
                }
            }
            
            return $dados;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosidePeriodo($nrorg, $nrinfoproctrab, $dtmescompini, $dtmescompfim, $nrvinculom, $codcateg, $nragentenocivo) {
        $sql = <<<SQL
/*getDadosS2500idePeriodo*/
SELECT TO_CHAR(CF.DTOCORRENCIA, 'YYYY-MM') perRef,
       SUM(DECODE(IC.NREVENTOM,4500,IC.VREVENTOFOLHA,0)) vrBcCpMensal,
       SUM(DECODE(IC.NREVENTOM,4505,IC.VREVENTOFOLHA,0)) vrBcCp13,
       0 vrBcFgts,
       0 vrBcFgts13,
       CASE 
        WHEN :P_CODCATEG LIKE '1%' OR :P_CODCATEG LIKE '2%' OR :P_CODCATEG LIKE '3%' OR :P_CODCATEG IN ('731', '734', '738') 
        THEN DECODE(:P_NRAGENTENOCIVO,NULL,1,:P_NRAGENTENOCIVO) 
        ELSE NULL 
       END grauExp,
       SUM(DECODE(IC.NREVENTOM,4510,IC.VREVENTOFOLHA,0)) vrBcFGTSProcTrab,
       SUM(DECODE(IC.NREVENTOM,4515,IC.VREVENTOFOLHA,0)) vrBcFGTSSefip,
       SUM(DECODE(IC.NREVENTOM,4520,IC.VREVENTOFOLHA,0)) vrBcFGTSDecAnt,
       '' codCateg,
       SUM(DECODE(IC.NREVENTOM,4525,IC.VREVENTOFOLHA,0)) vrBcCPrev
  FROM FPA_ITECALCFOLHA IC
  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IC.NRORG
                          AND CF.NRCALCULOFOLHA = IC.NRCALCULOFOLHA
                          AND CF.NRTIPOMOVIMENT = 100 /*PROCESSO TRABALHISTA*/
                          AND CF.NRTPMODALIDCAL = 1
                          AND CF.DTOCORRENCIA BETWEEN :P_DTMESCOMPINI AND :P_DTMESCOMPFIM
 WHERE IC.NRORG = :P_NRORG
   AND IC.NRVINCULOM = :P_NRVINCULOM
   AND IC.NREVENTOM IN (4500,4505,4510,4515,4520,4525)
 GROUP BY TO_CHAR(CF.DTOCORRENCIA, 'YYYY-MM')
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_NRINFOPROCTRAB" => $nrinfoproctrab,
            "P_DTMESCOMPINI" => $dtmescompini,
            "P_DTMESCOMPFIM" => $dtmescompfim,
            "P_NRVINCULOM" => $nrvinculom,
            "P_CODCATEG" => $codcateg,
            "P_NRAGENTENOCIVO" => $nragentenocivo,
        );
        return $this->connection->fetchAll($sql, $params);
    }
}