<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 01/09/2016
 * Time: 16:25
 */

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\VAB\Services\ParametroValeTransporteService;
use HCM\Util\DateUtil;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;

class ParametroValeTransporteController extends Crud {

    protected $dataSourceName_getParametroValeTransporte = 'getParametroValeTransporte';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ParametroValeTransporteService $parametroValeTransporteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->parametroValeTransporteService = $parametroValeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getParametroValeTransporte (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_getParametroValeTransporte);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function validaParamVale (DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as $key => &$row){
            $status = $this->parametroValeTransporteService->retornaUltHistoricoParamVale($row['NRORG'], $row['DTMESCOMPETENC']);

            if(!empty($status)){
                $response->addNotification(new Notification("Dados encontrados", Notification::TYPE_ERROR));
            } else {
                $response->addNotification(new Notification("Dados não encontrados", Notification::TYPE_SUCCESS));
            }
        }
    }
    
    public function deleteParametroVale(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $this->parametroValeTransporteService->deleteParametroVale($dataset);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );

        return $newFilterCriteria;
    }
    
}