DROP TYPE QUERY_VDITEM1_CMP99000_TABLE
/

DROP TYPE QUERY_VDITEM1_CMP99000_TYPE
/

CREATE TYPE QUERY_VDITEM1_CMP99000_TYPE AS
OBJECT(DSFILIAL          VARCHAR2(150),
       CDFILSOLI         VARCHAR2(4),
       CDPRODUTO         VARCHAR2(10),
       NMPRODUTO         VARCHAR2(50),
       CDARVPROD         VARCHAR2(20),
       DSERRO            VARCHAR2(2000),
       CDFORNECED        VARCHAR2(21),
       IDTPIJURFORN      VARCHAR2(1),
       NRINSJURFORN      VARCHAR2(20),
       NMRAZSOCFORN      VARCHAR2(100),
       NMFANTFORN        VARCHAR2(40),
       POSICAO           VARCHAR2(10));

/

CREATE TYPE QUERY_VDITEM1_CMP99000_TABLE AS TABLE OF QUERY_VDITEM1_CMP99000_TYPE
/

CREATE OR REPLACE FUNCTION QUERY_VDITEM1_CMP99000(P_DSFILIAL      IN VARCHAR2,
                                                  P_NRINSJURFORN  IN VARCHAR2,
                                                  P_CDARVPROD     IN VARCHAR2,
                                                  P_POSICAO       IN VARCHAR2,
                                                  P_QTSOLIPROD    IN FLOAT,
                                                  P_VRCUSTO       IN FLOAT
                                                  )RETURN QUERY_VDITEM1_CMP99000_TABLE PIPELINED
IS
   OUT_RECORD QUERY_VDITEM1_CMP99000_TYPE := QUERY_VDITEM1_CMP99000_TYPE (NULL, NULL, NULL, NULL, NULL, NULL,
                                                                          NULL, NULL, NULL, NULL, NULL, NULL);
   V_DSFILIAL          VARCHAR2(150) := P_DSFILIAL;
   V_CDFILSOLI         VARCHAR2(4);
   V_CDPRODUTO         VARCHAR2(10);
   V_NMPRODUTO         VARCHAR2(50);
   V_CDARVPROD         VARCHAR2(20) := P_CDARVPROD;
   V_DSERRO            VARCHAR2(2000) := '';
   V_CDFORNECED        VARCHAR2(21);
   V_IDTPIJURFORN      VARCHAR2(1);
   V_NRINSJURFORN      VARCHAR2(20) := P_NRINSJURFORN;
   V_NMRAZSOCFORN      VARCHAR2(100);
   V_NMFANTFORN        VARCHAR2(40);
   V_POSICAO           VARCHAR2(10);
   V_QTSOLIPROD        FLOAT;
   V_VRCUSTO           FLOAT;

BEGIN
   IF (NVL(P_CDARVPROD, 'W') <> 'W') THEN
      SELECT MAX(CDPRODUTO), MAX(NMPRODUTO), MAX(CDARVPROD)
        INTO V_CDPRODUTO, V_NMPRODUTO, V_CDARVPROD
        FROM PRODUTO
       WHERE CDARVPROD = P_CDARVPROD;
      IF (NVL(V_CDPRODUTO, 'W') = 'W') THEN
         V_DSERRO := V_DSERRO||'_'||'ERROCDARVPROD'; /*Produto não cadastrado.*/
      END IF;
   ELSE
      V_DSERRO := V_DSERRO||'_'||'ERROCDARVPRODV'; /*Produto de ser informado.*/
   END IF;
   IF (NVL(P_DSFILIAL, 'W') <> 'W') THEN
      SELECT MAX(CDFILIAL)
        INTO V_CDFILSOLI
        FROM FILIAL
       WHERE CASE
                WHEN LENGTH(FILIAL.NMCOMPRZSFIL||FILIAL.SGFILIAL) <> 0
                THEN LPAD(NVL(NMCOMPRZSFIL, '0'), 2, '0') || ' - ' || RPAD(SUBSTR(NVL(FILIAL.SGFILIAL, '    '), 1, 4), 4, ' ')|| ' - ' || FILIAL.NMFILIAL
                ELSE FILIAL.CDFILIAL || ' - ' || FILIAL.NMFILIAL
             END = P_DSFILIAL;
      IF (NVL(V_CDFILSOLI,'W') = 'W') THEN
         V_DSERRO := V_DSERRO||'_'||'ERROCDFILSOLI'; /*Unidade não cadastrada.*/
      END IF;
   ELSE
      V_DSERRO := V_DSERRO||'_'||'ERROCDFILSOLIV'; /*Unidade deve ser informada.*/
   END IF;
   IF (NVL(P_NRINSJURFORN, 'W') <> 'W') THEN
      SELECT MAX(CDFORNECED), MAX(IDTPIJURFORN), MAX(NRINSJURFORN),
             MAX(NMRAZSOCFORN),MAX(NMFANTFORN)
        INTO V_CDFORNECED, V_IDTPIJURFORN, V_NRINSJURFORN, V_NMRAZSOCFORN, V_NMFANTFORN
        FROM FORNECEDOR
       WHERE ((NRINSJURFORN =  P_NRINSJURFORN) OR (CDEXFORNECED = P_NRINSJURFORN));
      IF (NVL(V_CDFORNECED, 'W') = 'W') THEN
         V_DSERRO := V_DSERRO||'_'||'ERRONRINSJURFORN'; /*Fornececedor não cadastrado.*/
      END IF;
   ELSE
      V_DSERRO := V_DSERRO||'_'||'ERRONRINSJURFORNV'; /*Fornececedor de ser informado.*/
   END IF;

   IF (NVL(P_QTSOLIPROD, 0) = 0) THEN
      V_DSERRO := V_DSERRO||'_'||'ERROQTSOLIPRODZ'; /*Quantidade a comprar deve ser maior que zero.*/
   ELSE
      BEGIN
         SELECT TO_NUMBER(P_QTSOLIPROD)
           INTO V_QTSOLIPROD
           FROM DUAL;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_DSERRO := V_DSERRO||'_'||'ERROQTSOLIPROD'; /*Quantidade a comprar inválida.*/
         END;
      END;
   END IF;

   IF (NVL(P_VRCUSTO, 0) = 0) THEN
      V_DSERRO := V_DSERRO||'_'||'ERROVRCUSTOZ'; /*Custo do item a comprar deve ser maior que zero.*/
   ELSE
      BEGIN
         SELECT TO_NUMBER(P_VRCUSTO)
           INTO V_VRCUSTO
           FROM DUAL;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_DSERRO := V_DSERRO||'_'||'ERROVRCUSTO'; /*Custo do item a comprar inválido.*/
         END;
      END;
   END IF;
   OUT_RECORD.DSFILIAL          :=  P_DSFILIAL;
   OUT_RECORD.CDARVPROD         :=  P_CDARVPROD;
   OUT_RECORD.NRINSJURFORN      :=  P_NRINSJURFORN;
   OUT_RECORD.CDFILSOLI         :=  V_CDFILSOLI;
   OUT_RECORD.CDPRODUTO         :=  V_CDPRODUTO;
   OUT_RECORD.NMPRODUTO         :=  V_NMPRODUTO;
   OUT_RECORD.DSERRO            :=  V_DSERRO;
   OUT_RECORD.CDFORNECED        :=  V_CDFORNECED;
   OUT_RECORD.IDTPIJURFORN      :=  V_IDTPIJURFORN;
   OUT_RECORD.NMRAZSOCFORN      :=  V_NMRAZSOCFORN;
   OUT_RECORD.NMFANTFORN        :=  V_NMFANTFORN;
   OUT_RECORD.POSICAO           := P_POSICAO;
   PIPE ROW (OUT_RECORD);
END;
/